<?xml version="1.0" encoding="utf-8"?>

<!--
menu generation for kirsanov.com
written by dmitry@kirsanov.com
-->

<!DOCTYPE xsl:stylesheet [ <!ENTITY nbsp "&#160;">  ]>

<xsl:stylesheet
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.1"
    xmlns:saxon="http://icl.com/saxon"
    xmlns:dk="http://www.kirsanov.com/studio"
    extension-element-prefixes="saxon dk"
>

<xsl:output method="html" encoding="UTF-8" indent="yes"/> 

  <!--empty string, for comparisons-->
<xsl:variable name="emptystring"></xsl:variable>

<!--separator between menu items-->
<xsl:variable name="separator"><xsl:text> . </xsl:text></xsl:variable>

<!--amount by which separators are shifted vertically, em/1000-->
<xsl:variable name="shift-separators">-290</xsl:variable>

  <!--the name of the xml file where all site-wide information is stored (menu, copyrights etc)-->
<xsl:variable name="site">../_site.xml</xsl:variable>

  <!--name of the font file, referred to from svg, less the pfb extension-->
<xsl:variable name="font-filename">wmb_____</xsl:variable>

  <!--name of the metrics file, converted from afm by afm2xfm.awk-->
<xsl:variable name="xfm"><xsl:value-of select="$font-filename"/>.xfm</xsl:variable>

  <!--height of caps in the menu, in pixels (before resizing)-->
<xsl:variable name="targetcapheight">60</xsl:variable>

  <!--base character tracking in menu, in em/1000-->
<xsl:variable name="tracking">50</xsl:variable>

  <!--number of menu lines-->
<xsl:variable name="menulines">6</xsl:variable>

  <!--stretchability of the space chars-->
<xsl:variable name="space-stretchability">2.3</xsl:variable>

  <!--a perfect square of horizontal lines looks taller than wider, so we use a compensation coefficient to slightly squeeze it vertically -->
<xsl:variable name="square-compensation">0.96</xsl:variable>

<!--offset of the top left corner of the menu block on the canvas, in em/1000-->
<xsl:variable name="ini-x">1000</xsl:variable>
<xsl:variable name="ini-y">2000</xsl:variable>

<!--colors-->
<xsl:variable name="bg-color"><xsl:value-of select="document($site)//colors/color[@id='light']/text()"/></xsl:variable>  
<xsl:variable name="black">#000000</xsl:variable>
<xsl:variable name="white">#ffffff</xsl:variable>

  <!--the side of the menu square, to be calculated later-->
<xsl:variable saxon:assignable="yes" name="squareside">0</xsl:variable>

  <!--the distance between lines, to be calculated later-->
<xsl:variable saxon:assignable="yes" name="linespacing">0</xsl:variable>

  <!--the menu string, to be calculated later-->
<xsl:variable saxon:assignable="yes" name="sstring"></xsl:variable>

  <!--the font size to record into svg, to be calculated later-->
<xsl:variable saxon:assignable="yes" name="fontsize"></xsl:variable>

  <!--the scale coefficient for converting em/1000 units to (large) pixels-->
<xsl:variable saxon:assignable="yes" name="scale"></xsl:variable>

  <!--what we scale (large) pixels down to in order to fit into the 1500 by 1500 swf canvas-->
<xsl:variable saxon:assignable="yes" name="swf-scale">0.5</xsl:variable>

  <!--target width of the bitmap menu, for calculating imagemap-->
<xsl:variable saxon:assignable="yes" name="imp-squareside">185</xsl:variable>

<!--target width of the bitmap menu, for calculating imagemap, to be calculated later-->
<xsl:variable saxon:assignable="yes" name="imp-scale"></xsl:variable>

<!--margin of the swfml canvas-->
<xsl:variable saxon:assignable="yes" name="swf-margin">0</xsl:variable>

<!--temp vars for calculating clickable rectangles-->
<xsl:variable saxon:assignable="yes" name="x-tl"/>
<xsl:variable saxon:assignable="yes" name="y-tl"/>
<xsl:variable saxon:assignable="yes" name="x-br"/>
<xsl:variable saxon:assignable="yes" name="y-br"/>

<!--the total width of the unfilled string, to be calculated later-->
<xsl:variable saxon:assignable="yes" name="totalwidth"></xsl:variable>

  <!--the target length of one line, to be calculated later-->
<xsl:variable saxon:assignable="yes" name="linewidth"></xsl:variable>

<!--storage for the nodesets containing position information for all characters of the menu-->
<xsl:variable saxon:assignable="yes" name="tokens0"></xsl:variable>
<xsl:variable saxon:assignable="yes" name="tokens1"></xsl:variable>
<xsl:variable saxon:assignable="yes" name="tokens2"></xsl:variable>

<xsl:variable saxon:assignable="yes" name="fc">1</xsl:variable>

<!--including all character glyph templates-->
<xsl:include href="a.xsl"/>
<xsl:include href="b.xsl"/>
<xsl:include href="c.xsl"/>
<xsl:include href="d.xsl"/>
<xsl:include href="e.xsl"/>
<xsl:include href="f.xsl"/>
<xsl:include href="g.xsl"/>
<xsl:include href="h.xsl"/>
<xsl:include href="i.xsl"/>
<xsl:include href="j.xsl"/>
<xsl:include href="k.xsl"/>
<xsl:include href="l.xsl"/>
<xsl:include href="m.xsl"/>
<xsl:include href="n.xsl"/>
<xsl:include href="o.xsl"/>
<xsl:include href="p.xsl"/>
<xsl:include href="q.xsl"/>
<xsl:include href="r.xsl"/>
<xsl:include href="s.xsl"/>
<xsl:include href="t.xsl"/>
<xsl:include href="u.xsl"/>
<xsl:include href="v.xsl"/>
<xsl:include href="w.xsl"/>
<xsl:include href="x.xsl"/>
<xsl:include href="y.xsl"/>
<xsl:include href="z.xsl"/>


<xsl:template match="/">
  <!--information from the main input file is not used-->

<xsl:message>menu generation for kirsanov.com</xsl:message>
<xsl:message>written by dmitry@kirsanov.com</xsl:message>

  <!--we run across all sections and subsections of the menu and concatenate them into one sstring, using separators-->
<xsl:for-each select="document($site)//section | document($site)//subsection">
  <xsl:choose>
    <xsl:when test="$sstring != $emptystring">
      <saxon:assign name="sstring" select="concat($sstring, $separator, .)"/>
    </xsl:when>
    <xsl:otherwise>
      <saxon:assign name="sstring" select="."/>
    </xsl:otherwise>
</xsl:choose>
</xsl:for-each>

  <!--fontsize (px) divided by target cap height (px) equals 1000 (in em/1000 units) divided by capheight in the afm file (in em/1000 units)-->
<saxon:assign name="fontsize" select="($targetcapheight * 1000) div (document($xfm)//capheight)"/>
  <!--now that we know fontsize, we can calculate the scaling factor between em/1000 units and swfml pixels-->
<saxon:assign name="scale" select="$fontsize div 1000"/>

<!--we call the recursive template which will break the sstring into tokens stored in the variable token0-->
<xsl:call-template name="tokenize"><xsl:with-param name="str" select="$sstring"/></xsl:call-template>

<!--in this loop we copy all the elements from tokens0 to tokens1 adding some cumulative attribs as we go-->
<xsl:for-each select="$tokens0//*">
<saxon:assign name="tokens1">
   <xsl:copy-of select="$tokens1"/>
   <xsl:copy>
        <!--copying all existing attributes-->
        <xsl:for-each select="@*"><xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute></xsl:for-each>
       <!--the following 3 attribs are added-->
       <!--index is just the number of the char in the total string-->
     <xsl:attribute name="index"><xsl:number count="char"/></xsl:attribute>
       <!--dist is the distance of the char's left edge from the beginning of the string, in em/1000 (unfilled!)-->
     <xsl:attribute name="dist"><xsl:value-of select="sum(preceding-sibling::char/@width) + sum(preceding-sibling::char/@kern) + sum(preceding-sibling::char/@track)"/></xsl:attribute>
     <!--initially all chars are on line 1:-->
     <xsl:attribute name="line">1</xsl:attribute>
   </xsl:copy>
</saxon:assign>
</xsl:for-each>

  <!--the total width of the unfilled string-->
<saxon:assign name="totalwidth" select="$tokens1/char[position()=last()]/@dist + $tokens1/char[position()=last()]/@width"/>
  <!--the target length of one line-->
<saxon:assign name="linewidth" select="$totalwidth div $menulines"/>

<!--calling the recursive template for assigning @line values to all chars-->
<xsl:call-template name="linebreak"><xsl:with-param name="li" select="1"/></xsl:call-template>

  <!--in this loop we calcualate the longest of all lines-->
<xsl:variable saxon:assignable="yes" name="linestart" select="0"/>
<xsl:for-each select="$tokens1//*">
  <!--if we're at the end of line...-->
  <xsl:if test="(@line != 0) and ((following-sibling::char[1]/@line = 0) or (position() = last()))">
    <!--...and if the line that ended is longer than all before it...-->
    <xsl:if test="((@dist + @width) - $linestart) &gt; $squareside">
       <!--...assign that length to squareside-->
      <saxon:assign name="squareside" select="(@dist + @width) - $linestart"/>
    </xsl:if>
  </xsl:if>
  <!--if we're before the beginning of a line...-->
  <xsl:if test="(@line = 0) and (following-sibling::char[1]/@line != 0)">
      <!--...set the new value of linestart-->
    <saxon:assign name="linestart" select="following-sibling::char[1]/@dist"/>
   </xsl:if>
</xsl:for-each>

<!--linespacing is calculated so that we get a slightly imperfect square, taking into account square-compensation-->
<saxon:assign name="linespacing" select="($squareside*$square-compensation - $menulines * ($targetcapheight div $scale)) div ($menulines - 1) + ($targetcapheight div $scale)"/>

<!--calling the recursive template to assign @fill values to all chars-->
<xsl:call-template name="fill"><xsl:with-param name="li" select="1"/></xsl:call-template>

<!--in this loop we calculate actual x and y coordinates for each char, taking fill and linespacing into account-->
<saxon:assign name="tokens2"></saxon:assign>
<xsl:variable saxon:assignable="yes" name="x"/>
<xsl:variable saxon:assignable="yes" name="y"/>
<xsl:variable saxon:assignable="yes" name="l" select="0"/>
<xsl:for-each select="$tokens1//*">
<saxon:assign name="tokens2">
   <xsl:copy-of select="$tokens2"/>
   <xsl:copy>
        <!--copying all attributes-->
        <xsl:for-each select="@*"><xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute></xsl:for-each>
        <!--initializing values for a new line-->
        <xsl:if test="(@line != $l) and (@line != 0)">
          <saxon:assign name="l" select="@line"/>
          <saxon:assign name="x" select="$ini-x"/>
          <saxon:assign name="y" select="$ini-y + ($l - 1)*$linespacing"/>
       </xsl:if>
       <!--the following 2 attribs are added-->
       <xsl:attribute name="x">
         <xsl:value-of select="$x"/>
         <saxon:assign name="x" select="$x + (@width + @kern + @track + @fill)"/>
       </xsl:attribute>
       <xsl:attribute name="y">
         <xsl:choose>
           <xsl:when test="dk:in($separator, @which) = 1">
             <xsl:value-of select="$y + $shift-separators"/>
           </xsl:when>
           <xsl:otherwise>
             <xsl:value-of select="$y"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
    </xsl:copy>
  </saxon:assign>
</xsl:for-each>

<!--now we need to assign tokens2 back to tokens1 and initialize the former,  in preparation for the next run-->
<saxon:assign name="tokens1"><xsl:copy-of select="$tokens2"/></saxon:assign>
<saxon:assign name="tokens2"></saxon:assign>

<saxon:assign name="imp-scale" select="$imp-squareside div ($squareside * $scale)"/>

<!--uncomment the following if you want to see the $tokens1 nodeset
serialized onto output, e.g. for debugging-->
<!--
<xsl:copy-of select="$tokens1"/>
totalwidth: <xsl:value-of select="$totalwidth"/>
squareside: <xsl:value-of select="$squareside"/>
imp-scale: <xsl:value-of select="$imp-scale"/>
-->

<!--paint everything black-->
<xsl:call-template name="blacken"/>
<!--paint all subsections white-->
<xsl:for-each select="document($site)//subsection">
  <xsl:call-template name="whiten"><xsl:with-param name="what" select="."/></xsl:call-template>
</xsl:for-each>

<!--generating the svg file-->
<saxon:output href="menu.svg" method="html" omit-xml-declaration="yes">
  <svg width="2300px" height="2300px"> <!--just big enough to suffice-->
    <rect width="2800px" height="2800px" x="-100px" y="-100px" style="fill: {$bg-color}"/>
    <xsl:for-each select="$tokens1//*">
      <xsl:if test="@line != 0">
        <!--imagemagick allows referring to a type 1 font file with its name prefixed by @-->
        <text style="fill: {@color}; font-family: @{$font-filename}.pfb; font-size: {round($fontsize)}px;" x="{round(@x * $scale)}px" y="{round(@y * $scale)}px"><xsl:value-of select="@which"/></text>
      </xsl:if>
    </xsl:for-each>
  </svg>
  <xsl:text>
  </xsl:text>
</saxon:output>

<!--generating the imagemap-->
<saxon:output href="map.html" method="xml" indent="no" omit-xml-declaration="yes">
<!--uncomment the following to test the imagemap:-->
<!-- <img src="menu.gif" usemap="#menu"/>-->
  <map name="menu">
  <xsl:for-each select="document($site)//subsection">
   
    <!--calling the template that generates the area element(s) for this subsection-->
  <xsl:call-template name="findarea"><xsl:with-param name="what" select="."/><xsl:with-param name="href" select="@url"/><xsl:with-param name="start" select="string-length(substring-before($sstring, concat($separator, .)))+string-length($separator)"/><xsl:with-param name="end" select="string-length(substring-before($sstring, concat($separator, .)))+string-length($separator)+string-length(.)"/></xsl:call-template>
<!--calculating start and end is ugly, i will rewrite this to include start/end information in the tokens tree so the unreliable search can be removed-->

</xsl:for-each>
</map>
</saxon:output>

<!--calculating the tl and br corners of the menu rectangle for swfml-->
<xsl:for-each select="$tokens1//*">
<xsl:if test="not(preceding-sibling::*)">
  <saxon:assign name="x-tl" select="@x * $scale * $swf-scale - $swf-margin"/>
  <saxon:assign name="y-tl" select="(@y) * $scale * $swf-scale - $swf-margin"/>
</xsl:if>
<xsl:if test="not(following-sibling::*) and not(@line = 0)">
  <saxon:assign name="x-br" select="(@x + @width) * $scale * $swf-scale + $swf-margin"/>
  <saxon:assign name="y-br" select="((@y * $scale) + $targetcapheight) * $swf-scale + $swf-margin  "/>
</xsl:if>
</xsl:for-each>

<!--generating the swfml file-->
<saxon:output href="menu.swfml" method="html" omit-xml-declaration="yes">
  <!--the low rate allows the menu to behave more smoothly-->
  <swf w="{$x-br - $x-tl}" h="{$y-br - $y-tl}" color="{$bg-color}" rate="1">

<!--first we define shapes of all letters and the dot, -->
<!--each in coordinates 0,0 and black, to be scaled, moved and repainted on each use-->
    <xsl:for-each select="document($site)//alphabet/letter">
      <!--note that we need saxon's ability to calculate the name of the called template-->
      <xsl:call-template name="template-{.}" saxon:allow-avt="yes"/>
    </xsl:for-each>
    <!--we'll also need the ellipse for the dot in separator-->
    <ellipse ID='elli' x='0' y='0' w='1' h='1' color='FF000000'/>

    <xsl:for-each select="document($site)//subsection">

    <!--calling the template that generates the button element(s) for this subsection-->
      <xsl:call-template name="swf-findarea">
        <xsl:with-param name="what" select="."/>
        <xsl:with-param name="href" select="@url"/>
        <xsl:with-param name="id"><xsl:number level="multiple" count="subsection" from="root"/></xsl:with-param>
        <xsl:with-param name="frame"><xsl:number level="multiple" count="subsection" from="root"/></xsl:with-param>
        <xsl:with-param name="start"> 
          <xsl:choose>
            <xsl:when test="preceding-sibling::*[1][name()!='section']">
              <xsl:value-of select="string-length(substring-before($sstring, concat($separator, .)))+string-length($separator)"/>
            </xsl:when>
            <xsl:when test="not (preceding-sibling::*[2])">1</xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="string-length(substring-before($sstring, concat(preceding-sibling::*[1][name()='section'], $separator, .)))"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:with-param>
        <xsl:with-param name="end" select="string-length(substring-before($sstring, concat($separator, .)))+string-length($separator)+string-length(.)"/>
<!--calculating start and end is ugly, i will rewrite this to include start/end information in the tokens tree so the unreliable search can be removed-->
</xsl:call-template>

    </xsl:for-each>

  <!--paint everything black-->
  <xsl:call-template name="blacken"/>
<!--paint all subsections white-->
<xsl:for-each select="document($site)//subsection">
  <xsl:call-template name="whiten"><xsl:with-param name="what" select="."/></xsl:call-template>
</xsl:for-each>

<!--frame 0: default state (clickables white, nonclickables black)-->
<xsl:call-template name="swf-frame-ini"/>

<!--rest of frames: one of the subsections while, all others black-->
<xsl:for-each select="document($site)//subsection">
  <xsl:call-template name="blacken"/>
  <xsl:call-template name="whiten"><xsl:with-param name="what" select="."/></xsl:call-template>
  <xsl:call-template name="swf-frame-sub"/>
</xsl:for-each>

</swf>
<xsl:text>
</xsl:text>
</saxon:output>
</xsl:template>
<!--end of main, the rest is callable templates-->




<!--swf-frame-ini-->
<!--cycle to place all letters in the frame-->
<xsl:template name="swf-frame-ini">

  <xsl:for-each select="$tokens1//*">
  <xsl:if test="@line != 0">

  <xsl:choose>
      <!--if it's a dot, place an ellipse-->
  <xsl:when test="@which='.'">
    <PlaceObject depth='{@index}'>
       <xsl:attribute name="ID">elli</xsl:attribute>
       <Matrix tx='{dk:cx(@x) - 3}' ty='{dk:cy(@y) + 22}' sx='10' sy='10'/>

       <xsl:choose>
         <xsl:when test="@color=$white">
           <CxForm AddColor="00FFFFFF"/>
         </xsl:when>
         <xsl:otherwise>
           <CxForm MultColor="FF000000"/>
         </xsl:otherwise>
       </xsl:choose>

    </PlaceObject>
  </xsl:when>
    <!--if a space, do nothing-->
  <xsl:when test="@which=' '">
  </xsl:when>
    <!--if a letter, refer to its shape-->
  <xsl:otherwise>
    <PlaceObject depth="{@index}">
       <xsl:attribute name="ID"><xsl:value-of select="concat('shape-', translate(@which, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ', 'abcdefghijklmnopqrstuvwxyz'))"/></xsl:attribute>
      <xsl:choose>
        <xsl:when test="document(concat(@which, '.xsl'))//dk:shiftthis">
          <Matrix tx="{dk:cx(@x) + $swf-scale * 10 * document(concat(@which, '.xsl'))//dk:shiftthis/@tx}" ty="{dk:cy(@y) + $swf-scale * 10 * document(concat(@which, '.xsl'))//dk:shiftthis/@ty}" sx="{$swf-scale * 10}" sy="{$swf-scale * 10}"/>
        </xsl:when>
        <xsl:otherwise>
          <Matrix tx="{dk:cx(@x)}" ty="{dk:cy(@y)}" sx="{$swf-scale * 10}" sy="{$swf-scale * 10}"/>
        </xsl:otherwise>
      </xsl:choose>

       <xsl:choose>
         <xsl:when test="@color=$white">
           <CxForm AddColor="00FFFFFF"/>
         </xsl:when>
         <xsl:otherwise>
           <CxForm MultColor="FF000000"/>
         </xsl:otherwise>
       </xsl:choose>

    </PlaceObject>

  </xsl:otherwise>
  </xsl:choose>

  </xsl:if>
  </xsl:for-each>

  <DoAction><Stop/></DoAction>
  <ShowFrame>
    <xsl:attribute name="dk:fc"><xsl:value-of select="$fc"/></xsl:attribute>
    <saxon:assign name="fc" select="$fc+1"/>
  </ShowFrame>

</xsl:template>



<!--swf-frame-sub-->
<!--cycle over changeable letters only, referring by depth not ID-->
<xsl:template name="swf-frame-sub">

<xsl:for-each select="document($site)//subsection">

    <xsl:variable name="start" select="string-length(substring-before($sstring, concat($separator, .))) - 1 + string-length($separator)"/>
    <xsl:variable name="end" select="$start + string-length(.) + 4"/>
    <xsl:variable name="endindex" select="string-length($sstring) + 1"/>
    <xsl:variable saxon:assignable="yes" name="inside">no</xsl:variable>

    <xsl:for-each select="$tokens1//*">

          <xsl:choose>
            <!--start-->
            <xsl:when test="(@index = $start)">
              <saxon:assign name="inside">yes</saxon:assign>
            </xsl:when>
            <xsl:when test="(@index = $end) or (@index = $endindex)">
              <saxon:assign name="inside">no</saxon:assign>
            </xsl:when>
          </xsl:choose>

  <xsl:if test="(@line != 0) and ($inside = 'yes')">

  <xsl:choose>
      <!--if it's a dot, place an ellipse-->
  <xsl:when test="@which='.'">
    <PlaceObject depth='{@index}'>
       <xsl:choose>
         <xsl:when test="@color=$white">
           <CxForm AddColor="00FFFFFF"/>
         </xsl:when>
         <xsl:otherwise>
           <CxForm MultColor="FF000000"/>
         </xsl:otherwise>
       </xsl:choose>
    </PlaceObject>
  </xsl:when>

    <!--if a space, do nothing-->
  <xsl:when test="@which=' '">
  </xsl:when>

    <!--if a letter, refer to its shape-->
  <xsl:otherwise>
    <PlaceObject depth="{@index}">
       <xsl:choose>
         <xsl:when test="@color=$white">
           <CxForm AddColor="00FFFFFF"/>
         </xsl:when>
         <xsl:otherwise>
           <CxForm MultColor="FF000000"/>
         </xsl:otherwise>
       </xsl:choose>
    </PlaceObject>

  </xsl:otherwise>
  </xsl:choose>

  </xsl:if>
  </xsl:for-each>

  </xsl:for-each>

  <DoAction><Stop/></DoAction>
  <ShowFrame>
    <xsl:attribute name="dk:fc"><xsl:value-of select="$fc"/></xsl:attribute>
    <saxon:assign name="fc" select="$fc+1"/>
  </ShowFrame>

</xsl:template>



<!--swf-findarea-->
<!--generates an SWFML button corresponding to a portion of the string -->

<xsl:template name="swf-findarea">
  <xsl:param name="what"/>
  <xsl:param name="href"/>
  <xsl:param name="id"/>
  <xsl:param name="frame"/>
  <xsl:param name="start"/>
  <xsl:param name="end" saxon:assignable="yes"/>

  <xsl:variable name="x1" saxon:assignable="yes"></xsl:variable>
  <xsl:variable name="y1" saxon:assignable="yes"></xsl:variable>
  <xsl:variable name="x2" saxon:assignable="yes"></xsl:variable>
  <xsl:variable name="y2" saxon:assignable="yes"></xsl:variable>
  <xsl:variable saxon:assignable="yes" name="inside">no</xsl:variable>
  <xsl:variable saxon:assignable="yes" name="break-start">0</xsl:variable>
  <xsl:variable saxon:assignable="yes" name="break-end">0</xsl:variable>
  
    <xsl:for-each select="$tokens1//*">
      <xsl:choose>
        <xsl:when test="(@index = $start)">
          <saxon:assign name="inside">yes</saxon:assign>
          <saxon:assign name="x1" select="(following-sibling::node()/@x - ($linespacing div 4)) * $scale * $swf-scale"/>
          <xsl:choose>
            <xsl:when test="round((following-sibling::node()/@y - ($linespacing div 2)) * $scale  - $targetcapheight div 2) &lt; 0">
              <saxon:assign name="y1" select="0"/>
            </xsl:when>
            <xsl:otherwise>
              <saxon:assign name="y1" select="(following-sibling::node()/@y - ($linespacing div 2)) * $scale * $swf-scale"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>

        <xsl:when test="($inside = 'yes')">
          <xsl:choose>
            <xsl:when test="(@x &lt; preceding-sibling::*[1]/@x) and (@index &gt; $start + 1)">
              <saxon:assign name="inside">no</saxon:assign>
              <saxon:assign name="x2" select="(preceding-sibling::*[1]/@x + @width + ($linespacing div 4)) * $scale * $swf-scale"/>
              <xsl:choose>
                <xsl:when test="round((preceding-sibling::*[1]/@y + $linespacing div 2)  * $scale - $targetcapheight div 2) &lt; 0">
                  <saxon:assign name="y2" select="0"/>
                </xsl:when>
                <xsl:otherwise>
                  <saxon:assign name="y2" select="(preceding-sibling::*[2]/@y + $linespacing div 2)  * $scale * $swf-scale"/>
                </xsl:otherwise>
              </xsl:choose>
              <saxon:assign name="break-start" select="@index - 1 "/>
              <saxon:assign name="break-end" select="$end"/>
              <saxon:assign name="end" select="0"/>
            </xsl:when>

            <xsl:when test="(@index = $end)">
              <saxon:assign name="inside">no</saxon:assign>
              <saxon:assign name="x2" select="(@x + @width + ($linespacing div 4)) * $scale * $swf-scale"/>
              
              <xsl:choose>
                <xsl:when test="round((@y + $linespacing div 2)  * $scale - $targetcapheight div 2) &lt; 0">
                  <saxon:assign name="y2" select="0"/>
                </xsl:when>
                <xsl:otherwise>
                  <saxon:assign name="y2" select="(@y + $linespacing div 2)  * $scale * $swf-scale"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:when>
          </xsl:choose>
        </xsl:when>
      </xsl:choose>
</xsl:for-each>

<!--define the shape of the button:-->
<rect ID="buttonshape-{$id}" x="{$x1 - $x-tl}" y="{$y1 - $y-tl}" w="{$x2 - $x1}" h="{$y2 - $y1}" color="00000000"/>
<!-- create the button -->
<Button ID="button-{$id}" ismenu="0">
<OutFits>
<!-- all states refer to the same buttonshape -->
                     <OutFit state="OnIdle">
                             <ShapeRef ID="buttonshape-{$id}"/>
                         </OutFit>
                         <OutFit state="OnMouseOver">
                             <ShapeRef ID="buttonshape-{$id}"/>
                         </OutFit>
                         <OutFit state="OnMouseDown">
                             <ShapeRef ID="buttonshape-{$id}"/>
                         </OutFit>
                         <OutFit state="HitTest">
                             <ShapeRef ID="buttonshape-{$id}"/>
                         </OutFit>
                 </OutFits>
                 <ButtonActions>
                    <ButtonAction condition="IdleToOverUp">
                    <!-- on mouseover, go to the frame with the correspondingly painted menu -->
                      <GoToFrame frame="{$frame}"/>  
                    </ButtonAction>
                    <ButtonAction condition="OverUpToIdle">
                      <!-- on mouseout go to the default state in frame 0 -->
                                           <GoToFrame frame="0"/>  
                    </ButtonAction>
                    <ButtonAction condition="OverUpToOverDown">
                      <!-- on click, activate the url -->
                      <xsl:if test="$href != ''">
                        <GetUrl url="{$href}">  
                          <!--external links should open in new window-->
                          <xsl:if test="contains($href, '/')">
                            <xsl:attribute name="target">illustration</xsl:attribute>
                          </xsl:if>
                        </GetUrl>
                      </xsl:if>
                    </ButtonAction>
                 </ButtonActions>
               </Button>
<!-- place the button on the canvas -->
<PlaceObject ID="button-{$id}" depth="{10000+$id}"/>

<!--if the label is broken across lines, call itself recursively to generate a second button for the second half of the string-->
<xsl:if test="$break-start != 0">
  <xsl:call-template name="swf-findarea"><xsl:with-param name="start" select="$break-start"/><xsl:with-param name="end" select="$break-end"/><xsl:with-param name="what" select="$what"/><xsl:with-param name="href" select="$href"/><xsl:with-param name="id" select="100 + $id"/><xsl:with-param name="frame" select="$frame"/></xsl:call-template>
</xsl:if>

</xsl:template>


<!--findarea-->
<!--generates an HTML imagemap area corresponding to a portion of the string -->

<xsl:template name="findarea">

  <!--start and end indices of the substring in question-->
  <xsl:param name="start"/>
  <xsl:param name="end" saxon:assignable="yes"/>

  <!--the substring itself-->
  <xsl:param name="what"/>

  <!--where to link it to-->
  <xsl:param name="href"/>

  <!--here we'll store the comma-separated string for the coords attribute-->
  <xsl:variable name="areastring" saxon:assignable="yes"></xsl:variable>

  <!--flag showing if we are inside the substring when going through all the tokens-->
  <xsl:variable saxon:assignable="yes" name="inside">no</xsl:variable>

  <!--if the substring is broken, the second portion of it is stored here-->
  <xsl:variable saxon:assignable="yes" name="break-start">0</xsl:variable>
  <xsl:variable saxon:assignable="yes" name="break-end">0</xsl:variable>

  <xsl:if test="$start != 0">
    <area shape="rect" alt="{$what}" href="{$href}">

      <!--external links should open in new window-->
      <xsl:if test="contains($href, '/')">
        <xsl:attribute name="target">illustration</xsl:attribute>
      </xsl:if>

      <xsl:for-each select="$tokens1//*">
          <xsl:choose>

            <!--we're at the start of the substring-->
            <xsl:when test="(@index = $start) and ($inside = 'no')">
              <saxon:assign name="inside">yes</saxon:assign>
              <saxon:assign name="areastring" select="concat($areastring, string(round((following-sibling::*[1]/@x - $ini-x) * $scale * $imp-scale)), ',')"/>
              <xsl:choose>
                <xsl:when test="round((following-sibling::*[1]/@y - ($linespacing div 2)) * $scale  - $targetcapheight div 2) &lt; 0">
                  <saxon:assign name="areastring" select="concat($areastring, '0,')"/>
                </xsl:when>
                <xsl:otherwise>
                  <saxon:assign name="areastring" select="concat($areastring, string(round((((following-sibling::*[1]/@y - $ini-y) - $linespacing div 2) * $scale    + $targetcapheight div 2    ) * $imp-scale)), ',')"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:when>

            <!--we're inside the substring-->
            <xsl:when test="($inside = 'yes')">
              <xsl:choose>
                <!--if a break occurred,-->
                <xsl:when test="(@x &lt; preceding-sibling::*[1]/@x) and (@index &gt; $start + 1)">
                  <!--complete the current areastring-->
                  <saxon:assign name="inside">no</saxon:assign>
                  <saxon:assign name="areastring" select="concat($areastring, string(round((preceding-sibling::*[1]/@x + @width - $ini-x) * $scale * $imp-scale)), ',')"/>
                  <xsl:choose>
                    <xsl:when test="round((preceding-sibling::*[1]/@y - $linespacing div 2)  * $scale + $targetcapheight div 2) &lt; 0">
                      <saxon:assign name="areastring" select="concat($areastring, '0')"/>
                    </xsl:when>
                    <xsl:otherwise>
                      <saxon:assign name="areastring" select="concat($areastring, string(round((((preceding-sibling::*[1]/@y - $ini-y) + $linespacing div 2)  * $scale + $targetcapheight div 2) * $imp-scale)))"/>
                    </xsl:otherwise>
                  </xsl:choose>
                  <!--and set break indices in preparation for the recursive call-->
                  <saxon:assign name="break-start" select="@index - 1 "/>
                  <saxon:assign name="break-end" select="$end"/>
                  <saxon:assign name="end" select="0"/>
                </xsl:when>
                <!--if there was no break, but we reached the end of the subsrting-->
                <xsl:when test="(@index = $end)">
                  <saxon:assign name="inside">no</saxon:assign>
                  <saxon:assign name="areastring" select="concat($areastring, string(round((@x + @width - $ini-x) * $scale * $imp-scale)), ',')"/>
                  <xsl:choose>
                    <xsl:when test="round((@y - $linespacing div 2)  * $scale + $targetcapheight div 2) &lt; 0">
                      <saxon:assign name="areastring" select="concat($areastring, '0')"/>
                    </xsl:when>
                    <xsl:otherwise>
                      <saxon:assign name="areastring" select="concat($areastring, string(round((((@y - $ini-y) + $linespacing div 2)  * $scale + $targetcapheight div 2) * $imp-scale)))"/>
                    </xsl:otherwise>
                  </xsl:choose>
                </xsl:when>
              </xsl:choose>

            </xsl:when>
          </xsl:choose>
        </xsl:for-each>
        <!--emit the calculated areastring as coords attribute-->
        <xsl:attribute name="coords"><xsl:value-of select="$areastring"/></xsl:attribute>
      </area>
    </xsl:if>
    <xsl:if test="$break-start != 0">
  <!--if the substring was broken across lines, call itself recursively to generate a second area for the second half of the string-->
      <xsl:call-template name="findarea"><xsl:with-param name="start" select="$break-start"/><xsl:with-param name="end" select="$break-end"/><xsl:with-param name="what" select="$what"/><xsl:with-param name="href" select="$href"/></xsl:call-template>
    </xsl:if>
</xsl:template>


<!--blacken-->
<!--paints all the string black-->

<xsl:template name="blacken">
  <xsl:for-each select="$tokens1//*">
    <!--we copy from tokens1 to tokens2-->
    <saxon:assign name="tokens2">
      <xsl:copy-of select="$tokens2"/>
      <xsl:copy>
        <!--copying all attributes-->
        <xsl:for-each select="@*"><xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute></xsl:for-each>
        <xsl:attribute name="color"><xsl:value-of select="$black"/></xsl:attribute>
      </xsl:copy>
    </saxon:assign>
  </xsl:for-each>
  <!--now we need to assign tokens2 back to tokens1 and initialize the former,  in preparation for the next run-->
  <saxon:assign name="tokens1"><xsl:copy-of select="$tokens2"/></saxon:assign>
  <saxon:assign name="tokens2"></saxon:assign>
</xsl:template>

<!--whiten-->
<!--paints a portion of the string white-->

<xsl:template name="whiten">
  <xsl:param name="what"/>

  <!--this hack is ugly for sure, i will rewrite it when i have time-->
    <xsl:variable name="start" select="string-length(substring-before($sstring, concat($separator, $what)))+string-length($separator) - 2"/>
    <xsl:variable name="end" select="$start + string-length($what) + string-length($separator) + 1"/>
    
  <xsl:if test="$start != 0">
  <xsl:for-each select="$tokens1//*">
    <!--we copy from tokens1 to tokens2-->
    <saxon:assign name="tokens2">
      <xsl:copy-of select="$tokens2"/>
      <xsl:copy>
        <!--copying all attributes-->
        <xsl:for-each select="@*"><xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute></xsl:for-each>
        <xsl:attribute name="color">
          <xsl:choose>
            <!--if within the bounds, paint white-->
            <xsl:when test="(@index &gt;= $start) and (@index &lt;= $end)">
              <xsl:value-of select="$white"/>
            </xsl:when>
            <!--otherwise leave the existing value-->
            <xsl:otherwise>
              <xsl:value-of select="@color"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
      </xsl:copy>
    </saxon:assign>
  </xsl:for-each>
  </xsl:if>
  <!--now we need to assign tokens2 back to tokens1 and initialize the former,  in preparation for the next run-->
  <saxon:assign name="tokens1"><xsl:copy-of select="$tokens2"/></saxon:assign>
  <saxon:assign name="tokens2"></saxon:assign>
</xsl:template>

<!--fill-->
<!--recursive template to find & record fill values (additional kerns to fill lines) for all chars-->

<xsl:template name="fill">
    <!--li is the number of the current line for which we're calculating the fills:-->
  <xsl:param name="li"/>
    <!--if we are not beyond the last line, i.e. we still have at least the current line to fill:-->
  <xsl:if test="$li &lt;= $menulines">
   <xsl:variable saxon:assignable="yes" name="space" select="document($xfm)//char[@which='space']/@width"/>
  <xsl:variable saxon:assignable="yes" name="linelength" select="0"/>
  <xsl:variable saxon:assignable="yes" name="spaces" select="0"/>
  <xsl:variable saxon:assignable="yes" name="chars" select="0"/>
  <xsl:for-each select="$tokens1//*[@line=$li]">
    <saxon:assign name="chars" select="$chars + 1"/>
    <xsl:if test="@which=' '"><saxon:assign name="spaces" select="$spaces + 1"/></xsl:if>
    <xsl:if test="position() = last()"></xsl:if>
  </xsl:for-each>
  <saxon:assign name="linelength" select="($tokens1//char[@line = $li][$chars]/@dist) + ($tokens1//char[@line = $li][$chars]/@width) - ($tokens1//char[@line = $li][1]/@dist)"/>
  <xsl:for-each select="$tokens1//*">
    <!--we copy from tokens1 to tokens2-->
    <saxon:assign name="tokens2">
      <xsl:copy-of select="$tokens2"/>
      <xsl:copy>
        <!--copying all attributes-->
        <xsl:for-each select="@*"><xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute></xsl:for-each>
        <xsl:attribute name="fill">
          <xsl:choose>
            <!--if spaces suffice to fill the line...-->
              <xsl:when test="((($squareside - $linelength) div $spaces) div $space) + 1 &lt;= $space-stretchability">
                 <xsl:choose>
                   <!--if we're within the line and on a space, add fill-->
                   <xsl:when test="(@line = $li) and (@which = ' ')">
                     <xsl:value-of select="(($squareside - $linelength) div $spaces)"/>
                   </xsl:when>
                   <!--otherwise just use the existing value-->
                   <xsl:otherwise>
                     <xsl:value-of select="@fill"/>
                   </xsl:otherwise>
                 </xsl:choose>
             </xsl:when>
             <!--if spaces alone do not suffice-->
             <xsl:otherwise>
               <xsl:variable name="space-fill" select="($space * $space-stretchability) - $space"/>
               <xsl:variable name="all-fill" select="(($squareside - $linelength) - ($space-fill * $spaces)) div ($chars - 1)"/>
                 <xsl:choose>
                   <!--if we're within the line and on a space, add both space-fill and all-fill-->
                   <xsl:when test="(@line = $li) and (@which = ' ')">
                     <xsl:value-of select="$all-fill + $space-fill"/>
                    </xsl:when>
                    <xsl:otherwise>
                         <xsl:choose>
                           <!--if we're in line, add all-fill-->
                           <xsl:when test="@line = $li">
                             <xsl:value-of select="$all-fill"/>
                           </xsl:when>
                         <!--otherwise copy existing value-->
                         <xsl:otherwise>
                           <xsl:value-of select="@fill"/>
                         </xsl:otherwise>
                       </xsl:choose>
                     </xsl:otherwise>
                   </xsl:choose>                  
                 </xsl:otherwise>
               </xsl:choose>
             </xsl:attribute>
           </xsl:copy>
         </saxon:assign>
       </xsl:for-each>
  <!--now we need to assign tokens2 back to tokens1 and initialize the former,  in preparation for the next run-->
  <saxon:assign name="tokens1"><xsl:copy-of select="$tokens2"/></saxon:assign>
  <saxon:assign name="tokens2"></saxon:assign>
  <!-- now we re-curse ourselves :) -->
   <xsl:call-template name="fill"><xsl:with-param name="li" select="$li + 1"/></xsl:call-template>
</xsl:if>
</xsl:template>

<!--linebreak-->
<!--the recursive template to find & record closest possible linebreaks-->

<xsl:template name="linebreak">
    <!--li is the number of the current line the break _after_ which we're calculating:-->
  <xsl:param name="li"/>
    <!--if we are not on the last line, i.e. still need to find more breaks:-->
  <xsl:if test="$li &lt; $menulines">
    <!--we calculate the target point where we would like the line to break:-->
  <xsl:variable name="target" select="$li * $linewidth"/>
    <!--first we loop to find the linebreak point closest to our target point:-->
  <xsl:variable saxon:assignable="yes" name="closest" select="10000000"/>
  <xsl:variable saxon:assignable="yes" name="index-closest" select="0"/>
  <xsl:for-each select="$tokens1//*">

      <!--breaks allowed only before separators:-->
    <xsl:if test="(dk:in($separator, @which) = 0) and (dk:in($separator, following-sibling::char[1]/@which) = 1)">

      <!--if the current point is closer to the target than the previous...-->
    <xsl:if test="dk:abs((@dist + @width) - $target) &lt; $closest">
      <!--...remember it-->
      <saxon:assign name="closest" select="dk:abs((@dist + @width) - $target)"/>
      <saxon:assign name="index-closest" select="@index"/>
    </xsl:if>
    </xsl:if>
  </xsl:for-each>
    <!--now we loop to assign the line attribute to all chars.-->
    <!--the newline variable is needed to exclude spaces right after the break-->
  <xsl:variable saxon:assignable="yes" name="newline" select="0"/>
  <xsl:for-each select="$tokens1//*">
    <!--we copy from tokens1 to tokens2-->
    <saxon:assign name="tokens2">
      <xsl:copy-of select="$tokens2"/>
      <xsl:copy>
        <!--copying all attributes-->
        <xsl:for-each select="@*"><xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute></xsl:for-each>
        <xsl:attribute name="line">
          <xsl:choose>
            <!--for points before the break, just copy their existing value of @line-->
            <xsl:when test="@index &lt;= $index-closest"><xsl:value-of select="@line"/></xsl:when>
            <xsl:otherwise>
               <xsl:choose>
                 <!--if newline is still zero, we're at a separator following the break, so we discard it by assigning @line=0-->
                 <xsl:when test="$newline = 0">
                   <xsl:value-of select="0"/>
                   <!--and if next char is not separator, assign newline=1-->
                   <xsl:if test="(dk:in($separator, following-sibling::char[1]/@which) = 0)"><saxon:assign name="newline" select="1"/></xsl:if>
                 </xsl:when>
                <xsl:otherwise>
                   <!--for all other chars, we assign @line=li+1 -->
                   <xsl:value-of select="$li + 1"/>
                </xsl:otherwise>
               </xsl:choose>  
           </xsl:otherwise>
         </xsl:choose>  
       </xsl:attribute>
   </xsl:copy>
    </saxon:assign>
  </xsl:for-each>
  <!--now we need to assign tokens2 back to tokens1 and initialize the former,  in preparation for the next run-->
  <saxon:assign name="tokens1"><xsl:copy-of select="$tokens2"/></saxon:assign>
  <saxon:assign name="tokens2"></saxon:assign>
  <!-- now we re-curse ourselves :) -->
   <xsl:call-template name="linebreak"><xsl:with-param name="li" select="$li + 1"/></xsl:call-template>
</xsl:if>
</xsl:template>

<!--tokenize-->
<!--recursively goes through str and records its chars into tokens0-->

<xsl:template name="tokenize">
  <xsl:param name="str"/>
  <xsl:if test="$str != $emptystring">
    <saxon:assign name="tokens0">
      <xsl:copy-of select="$tokens0"/>
      <xsl:variable name="thischar" select="dk:charname(substring($str,1,1))"/>
      <xsl:variable name="nextchar" select="dk:charname(substring($str,2,1))"/>
      <char>
        <xsl:attribute name="which"><xsl:value-of select="substring($str,1,1)"/></xsl:attribute>
        <xsl:attribute name="width"><xsl:value-of select="document($xfm)//char[@which=$thischar]/@width"/></xsl:attribute>
        <xsl:attribute name="kern">
          <xsl:choose>
            <xsl:when test="document($xfm)//kern[@left=$thischar][@right=$nextchar]">
              <xsl:value-of select="document($xfm)//kern[@left=$thischar][@right=$nextchar]/@kern"/>
            </xsl:when>
            <xsl:otherwise>0</xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
        <xsl:attribute name="track"><xsl:value-of select="$tracking"/></xsl:attribute>
        <xsl:attribute name="fill"><xsl:value-of select="0"/></xsl:attribute>
        <xsl:attribute name="color"><xsl:value-of select="$white"/></xsl:attribute>
      </char>
    </saxon:assign>
    <!-- now we re-curse ourselves :) -->
    <xsl:call-template name="tokenize"><xsl:with-param name="str" select="substring($str,2)"/></xsl:call-template>
  </xsl:if>
</xsl:template>

<!--dk:charname-->
<!--returns the postscript name of the character (the one used in the afm file)-->

<saxon:function name="dk:charname">
  <xsl:param name="char"/>
  <saxon:return>
    <xsl:choose>
      <xsl:when test="$char=' '">space</xsl:when>
      <xsl:when test="$char=','">comma</xsl:when>
      <xsl:when test="$char='.'">period</xsl:when>
      <xsl:when test="$char='&#183;'">bullet</xsl:when>
      <xsl:otherwise><xsl:value-of select="$char"/></xsl:otherwise>
    </xsl:choose>
  </saxon:return>
</saxon:function>

<!--dk:abs-->
<!--returns the absolute value-->

<saxon:function name="dk:abs">
  <xsl:param name="num"/>
  <saxon:return>
    <xsl:choose>
      <xsl:when test="$num &lt; 0"><xsl:value-of select="-$num"/></xsl:when>
      <xsl:otherwise><xsl:value-of select="$num"/></xsl:otherwise>
    </xsl:choose>
  </saxon:return>
</saxon:function>

<!--dk:in-->
<!--returns true if the second argument string is within the first, otherwise false-->
<saxon:function name="dk:in">
  <xsl:param name="s1"/>
  <xsl:param name="s2"/>
  <saxon:return>
    <xsl:choose>
      <xsl:when test="((substring-before($s1, $s2) = $emptystring) and (substring-after($s1, $s2) = $emptystring)) or ($s2 = $emptystring)"><xsl:value-of select="0"/></xsl:when>
      <xsl:otherwise><xsl:value-of select="1"/></xsl:otherwise>
    </xsl:choose>
  </saxon:return>
</saxon:function>

<!--these two functions return scaled values of coordinates, used in generating swf frames-->

<saxon:function name="dk:cx">
  <xsl:param name="x"/>
  <saxon:return>
    <xsl:value-of select="($x * $scale * $swf-scale) - $x-tl"/>
  </saxon:return>
</saxon:function>

<saxon:function name="dk:cy">
  <xsl:param name="y"/>
  <saxon:return>
    <xsl:value-of select="($y * $scale * $swf-scale) - $y-tl"/>
  </saxon:return>
</saxon:function>

</xsl:stylesheet>
