<?xml version="1.0" encoding="utf-8"?>

<!--
page generation for kirsanov.com
written by alina@kirsanov.com

Requires files:

     ./_site.xml must contain:

         a menu/section/subsection tree with link URLs (in
         subsection/@url) and menu labels (in subsection/text())

         colors/color[@id="light"] and colors/color[@id="dark"] 
         color values

         buttons/button texts for different small buttons used on pages 

         copyright, description, and keywords texts

     ./menu/map.html, ./menu/menu.gif, ./menu/menu.swf for menu

     a ./*.xml source file for each section

     ./thumbs/*.png to generate 103x103-pixels thumbnails for section pages   

     ./display/* to generate separate display html pages with large
     images

     ./mybo____.ttf, ./pp-bold_.ttf and ./pp______.ttf font files used 
     to generate text buttons

     com.kirsanov.graphics.getsize, com.kirsanov.graphics.getsize$1, 
     com.kirsanov.runtime.run, com.kirsanov.utils.RandomClass java classes

Generates files:

     html pages for sections 

     html display pages with large images

     ./img/*: headings, textual buttons, arrows, backgrounds 
     used for thumbnail generation, four thumbnails with different 
     backgrounds for each ./thumb/ image  
-->                                                                                                                                     

<!DOCTYPE xsl:stylesheet [ 
  <!ENTITY nbsp "&#160;"> 
  <!ENTITY mdash "&#8212;">
]>

<xsl:stylesheet
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.1"
   xmlns:saxon="http://icl.com/saxon"
   xmlns:dk-g="com.kirsanov.graphics.getsize"
   xmlns:dk-r="com.kirsanov.runtime.run"
   xmlns:dk-u="com.kirsanov.utils.RandomClass"
   extension-element-prefixes="saxon dk-g dk-r dk-u"
>

<xsl:output method="html" encoding="utf-8" indent="no" saxon:omit-meta-tag="yes" doctype-public="-//W3C//DTD HTML 4.0 Transitional//EN"/>

<!-- command-line parameter to turn image generation on -->
<xsl:param name="gen"/>

<!-- to be edited after installation: -->

<!-- path to the ImageMagick folder -->     
<xsl:variable name="im-path">C:/ImageMagick/ImageMagick-win2k/</xsl:variable>
<!-- path to the source folder --> 
<xsl:variable name="s-path">/00/stu/00/</xsl:variable>
<!-- fonts for graphic generation --> 
<xsl:variable name="font-button">mybo____.ttf</xsl:variable>
<xsl:variable name="font-light">pp______.ttf</xsl:variable>
<xsl:variable name="font-bold">pp-bold_.ttf</xsl:variable>

<!-- other constants: -->

<!-- file with all the needed info for page/image generation -->
<xsl:variable name="site">_site.xml</xsl:variable> 
<!-- name of the current source file -->
<xsl:variable name="current" select="substring-before(substring-after(saxon:systemId(), $s-path), '.xml')"/>
<!-- two main colors --> 
<xsl:variable name="dark" select="document($site)//colors/color[@id='dark']/text()"/>
<xsl:variable name="light" select="document($site)//colors/color[@id='light']/text()"/>
<!-- symbols used in XPath expressions -->
<xsl:variable name="apostrophe">'</xsl:variable> 
<xsl:variable name="quote">"</xsl:variable>  
<!-- value of dissolve for thumb generation -->
<xsl:variable name="dissolve" select="25"/> 
<!-- space between columns in a  block --> 
<xsl:variable name="w-btw-col" select="23"/>
<!-- space between blocks -->  
<xsl:variable name="w-btw-block" select="68"/>
<!-- space between an arrow and the corresponding column: -->  
<xsl:variable name="w-btw-arrow" select="10"/> 
<!-- column width: -->
<xsl:variable name="w-col" select="156"/>
<!-- font size of the main heading: -->  
<xsl:variable name="main-fs" select="64"/>  
<!-- font size of top headings: --> 
<xsl:variable name="top-fs" select="25"/>  
<!-- font size of small buttons: --> 
<xsl:variable name="but-fs" select="9"/>  

<!-- assignable variables: -->

<!-- number of the vertical position of the thumb preceding the current one;
     used to be avoid the situation when two adjacent thumbs are placed
     on the same vertical level -->
<xsl:variable name="prev" select="1" saxon:assignable="yes"/>

<!-- page skeleton -->
<xsl:template match="page">
<xsl:call-template name="credits"/>
<html>
<head>
<title> 
<xsl:if test="$current != 'index'"><xsl:value-of select="title/text()"/><xsl:text> - </xsl:text></xsl:if>
<xsl:value-of select="document($site)//title"/>
</title>
<meta name="description" content="{document($site)//content}"/>
<meta name="keywords" content="{document($site)//keywords}"/>
<xsl:call-template name="css"/>
<xsl:call-template name="script"/>
</head>
<body bgcolor="{$light}" background="img/bg-r.gif" text="#000000" link="#ffffff" vlink="#ffffff" alink="#ffffff" style="margin-top: 0px; margin-left: 0px; padding-top: 0px; padding-left: 0px;">
<!-- the content of a page is placed into a two-column table:
     the left column with a special background image 
     (with logo, studio name and rules) contains the beginning of a page
     (intro and the first block) which would be enough to fill the area
     with this background image; the right column contains the rest of info -->
<table cellspacing="0" cellpadding="0" border="0" width="100%">
<tr valign="top"><td style="background-image: url(img/bg-l.gif)"><table cellspacing="0" cellpadding="0" border="0">
<!-- the inner structure of left and right columns for index.html page
     and section pages is quite different -->
<xsl:choose>
  <xsl:when test="$current='index'"><xsl:call-template name="left-home"/></xsl:when>
  <xsl:otherwise><xsl:call-template name="left-page"/></xsl:otherwise>
</xsl:choose>
</table></td>
<td><table cellspacing="0" cellpadding="0" border="0">
<xsl:choose>
  <xsl:when test="$current='index'"><xsl:call-template name="right-home"/></xsl:when>
  <xsl:otherwise><xsl:call-template name="right-page"/></xsl:otherwise>
</xsl:choose>
</table></td>
</tr></table>
<xsl:if test="$current='index'">
<!-- for index.html, we place here four rotated images with absolute coordinates -->
<xsl:apply-templates select="image"/>
<!-- all the shared images (backgrounds, arrows, small textual buttons) 
     are generated when we run index.xml -->
  <xsl:if test="$gen='yes'">
<!-- generating arrows and one- or two-colored backgrounds for thumbs -->
    <xsl:apply-templates select="document($site)//colors/color"/>
<!-- generating small textual buttons -->
    <xsl:apply-templates select="document($site)//buttons/button"/>
<!-- generating four backgrounds with shadows for thumbs -->
    <xsl:call-template name="make-bg"/>
  </xsl:if>
</xsl:if>
<!-- imagemap info for browsers without flash support -->
<xsl:copy-of select="document('menu/map.html')"/>
</body>
</html> 
</xsl:template>

<!-- index.html -->
<xsl:template name="left-home">
<tr valign="top"><td style="background-image: none"><img alt="" src="img/e.gif" width="1" height="242"/><br/>
<!-- main title -->
<img alt="" src="img/e.gif" width="91" height="1"/><img alt="PredictablyPerfect.com: Dmitry Kirsanov Studio" src="img/ppc.gif" width="{dk-g:getw('img/ppc.gif')}" height="{dk-g:geth('img/ppc.gif')}"/><br/>
<img alt="" src="img/e.gif" width="1" height="83"/><br/>
<!-- flash or gif square menu -->
<img alt="" src="img/e.gif" width="380" height="1"/><xsl:call-template name="flash"/><br/>
<img alt="" src="img/e.gif" width="1" height="8"/></td></tr>
</xsl:template>

<xsl:template name="right-home">
<!-- we place no info in the right column for index.html -->
<tr valign="top"><td/></tr>
</xsl:template>

<!-- we put here the info for each frame inside the image;
     for each image, the number of frames should be the same
     and rotate the frames -->
<xsl:template match="image"><xsl:apply-templates/><script language="JavaScript" type="text/javascript"><xsl:text>
</xsl:text><xsl:comment><xsl:text>
</xsl:text>r(<xsl:value-of select="position()"/>,1);<xsl:text>
</xsl:text>//</xsl:comment><xsl:text>
</xsl:text></script>
</xsl:template>

<!-- for each frame, id is calculated from position of the image and 
     position of the frame inside it -->
<xsl:template match="frame"><span id="i{count(preceding::image) + 1}-{position() div 2}"><a href="{@href}" onmouseover="s({count(preceding::image) + 1})" onmouseout="r({count(preceding::image) + 1},{position() div 2})"><img width="128" height="126" border="0" src="img/i-{text()}-{parent::*/@bg}.gif" alt=""/></a></span></xsl:template>

<!-- first, we mark all the frames as hidden; 
     later, one frame of an image would be made visible at a time -->
<xsl:template mode="script" match="frame">document.getElementById("i"+i+"-<xsl:value-of select="position()"/>").style.visibility="hidden";</xsl:template>

<!-- for each frame, we use separate css style according to frame's id;  
     absolute position for each frame inside an image is the same;
     z-indeces differ for each frame -->
<xsl:template mode="style" match="frame">#i<xsl:value-of select="count(preceding::image) + 1"/>-<xsl:value-of select="count(preceding-sibling::frame) + 1"/> {position: absolute; left: <xsl:value-of select="parent::image/@left"/>; top: <xsl:value-of select="parent::image/@top"/>; z-index: <xsl:value-of select="count(preceding::image) + 1"/><xsl:value-of select="count(preceding-sibling::frame) + 1"/>}</xsl:template>

<!-- left column of the main table for section subpage -->
<xsl:template name="left-page">
<tr valign="top"><td style="background-image: none"><table cellspacing="0" cellpadding="0" border="0" style="background-image: none">
<tr valign="top"><td><table cellspacing="0" cellpadding="0" border="0"> 
<tr><td><img alt="" src="img/e.gif" width="1" height="9"/><br/>
<img alt="" src="img/e.gif" width="359" height="1"/></td>
<td><img alt="" src="img/e.gif" width="66" height="1"/></td>
<td><img alt="" src="img/e.gif" width="14" height="1"/></td></tr>
<tr valign="top">
<td colspan="3" align="right">
<!-- some section pages may contain a top thumb image; 
     if there is no top image we need to put a vertical strut instead -->
<xsl:choose>
  <xsl:when test="topimg">
    <xsl:apply-templates select="topimg"/>
  </xsl:when>
  <xsl:otherwise>
    <img alt="" src="img/e.gif" width="1" height="126"/>
  </xsl:otherwise>
</xsl:choose>
</td>
</tr>
<tr><td><img alt="" src="img/e.gif" width="1" height="101"/></td></tr>
<tr valign="top">
<td align="right">
<table cellspacing="0" cellpadding="0" border="0">
<tr valign="top">
<td><img alt="" src="img/e.gif" width="1" height="137"/></td>
<!-- main title-->
<td><xsl:apply-templates select="title"/><img alt="" src="img/e.gif" width="24" height="1"/></td>
</tr>
</table></td>
</tr>
<tr valign="bottom">
<!-- flash or gif square menu -->
<td align="right"><xsl:call-template name="flash"/><br/><img alt="" src="img/e.gif" width="1" height="8" border="0"/></td>
<!-- link to home page -->
<td align="center"><a href="index.html"><img alt="" src="img/at-light.gif" width="{dk-g:getw('img/at-light.gif')}" height="{dk-g:geth('img/at-light.gif')}" border="0"/><br/>
<img alt="Home" src="img/l-home.gif" width="{dk-g:getw('img/l-home.gif')}" height="{dk-g:geth('img/l-home.gif')}" border="0"/></a></td>
</tr>
</table></td>
<!-- only intro and first block of text are placed in the left column of 
     the main table -->
<xsl:apply-templates select="intro"/>
<xsl:apply-templates select="block[1]"/>
</tr></table></td></tr>
</xsl:template>

<xsl:template match="page/title">
<xsl:variable name="src" select="concat($s-path, 'img/hh-', $current)"/>
<!-- generating new title image -->
<xsl:if test="$gen='yes'">
  <!-- outputting the name of the currently processed file -->
  <xsl:message>generate: <xsl:value-of select="$src"/></xsl:message>
<!-- we create an svg file for a title from the processed file name. 
     the size is 4 times larger for better anti-aliasing. -->
  <xsl:document href="{$src}.svg" encoding="utf-8" method="xml" omit-xml-declaration="yes" indent="yes">
    <svg width="1600px" height="400px">
      <rect width="1800px" height="403px" x="-100px" y="-100px" style="fill: {$dark}"/>
      <rect width="1800px" height="196px" x="-100px" y="304px" style="fill: {$light}"/>
      <text style="fill: #ffffff; font-family: @{$font-light}; font-size: {$main-fs * 4}pt;" x="100px" y="276px"><xsl:value-of select="."/></text>
    </svg><xsl:text>
    </xsl:text>
  </xsl:document>
<!-- we convert svg image to png format, scale it 4 times down and
     convert to gif with 256 colors. 
     the background red color is made transparent.
     then we convert it to 16 colors. -->   
  <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert ', $src, '.svg ', $src, '.png'))"/></xsl:message>  
  <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify +dither -filter Box -geometry 25%x25% ', $src, '.png'))"/></xsl:message>
  <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert +dither ', $src, '.png ', $src, '.gif'))"/></xsl:message>   
  <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify +dither -fill ', $dark, ' -opaque ', $light, ' ', $src, '.gif'))"/></xsl:message>   
  <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify +dither -transparent ', $dark, ' ', $src, '.gif'))"/></xsl:message>   
  <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify +dither -colors 16 ', $src, '.gif'))"/></xsl:message>   
  <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -crop 0x0 ', $src, '.gif'))"/></xsl:message>   
<!-- we delete temporary files-->  
  <xsl:message><xsl:value-of select="dk-r:delete(concat($src, '.png'))"/></xsl:message>   
  <xsl:message><xsl:value-of select="dk-r:delete(concat($src, '.svg'))"/></xsl:message>   
</xsl:if>
<img alt="{.}" src="img/hh-{$current}.gif" width="{dk-g:getw(concat($src, '.gif'))}" height="{dk-g:geth(concat($src, '.gif'))}"/>
</xsl:template> 

<xsl:template match="topimg">
<!-- some section pages may contain a top thumb image 
     with some small text nearby 
     which may be either linked to a separate displaying page or url,
     or not linked at all --> 
<xsl:if test="$gen='yes'">
<!-- generating the corresponding thumb image -->
  <xsl:call-template name="thumb"/>
</xsl:if>
<xsl:variable name="src">
  <xsl:choose>
    <xsl:when test="@say = 'coming soon'">
     <xsl:value-of select="concat($s-path, 'img/l-comingsoon')"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="concat($s-path, 'img/l-', @say)"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:variable>
<table cellspacing="0" cellpadding="0" border="0">
  <tr valign="top">
    <xsl:choose>
      <xsl:when test="@display">
<!-- linked to a separate displaying page -->
        <td>
          <a href="{@display}.html">
            <xsl:call-template name="target">
              <xsl:with-param name="target" select="1"/>
            </xsl:call-template>
            <img alt="" src="img/e.gif" width="1" height="10" border="0"/><br/>
            <img alt="{@say}" src="img/l-{@say}.gif" width="{dk-g:getw(concat($src, '.gif'))}" height="{dk-g:geth(concat($src, '.gif'))}" border="0"/>
          </a>
        </td>
        <td>
          <a href="{@display}.html">
            <xsl:call-template name="target">
              <xsl:with-param name="target" select="1"/>
            </xsl:call-template>
            <img alt="" src="img/i-{@id}-1.gif" width="128" height="126" border="0"/>
          </a>
          <img alt="" src="img/e.gif" width="32" height="1"/>
        </td>
      </xsl:when>
      <xsl:when test="@href">
<!-- linked to a url -->
        <td>
          <a href="{@href}">
            <xsl:call-template name="target"/>
            <img alt="" src="img/e.gif" width="1" height="10" border="0"/><br/>
            <img alt="{@say}" src="img/l-{@say}.gif" width="{dk-g:getw(concat($src, '.gif'))}" height="{dk-g:geth(concat($src, '.gif'))}" border="0"/>
          </a>
        </td>
        <td>
          <a href="{@href}">
            <xsl:call-template name="target"/>
            <img alt="" src="img/i-{@id}-1.gif" width="128" height="126" border="0"/>
          </a>
          <img alt="" src="img/e.gif" width="32" height="1"/>
        </td>
      </xsl:when>
      <xsl:otherwise>
<!-- not linked -->
        <td>
          <img alt="" src="img/e.gif" width="1" height="10" border="0"/><br/>
          <img alt="{@say}" src="{substring-after($src, $s-path)}.gif" width="{dk-g:getw(concat($src, '.gif'))}" height="{dk-g:geth(concat($src, '.gif'))}" border="0"/>
        </td>
        <td>
          <img alt="" src="img/i-{@id}-1.gif" width="128" height="126"/><img alt="" src="img/e.gif" width="32" height="1" border="0"/>
        </td>
      </xsl:otherwise>
    </xsl:choose>
  </tr>
</table>
</xsl:template> 

<!-- right column of the main table for section subpage -->
<xsl:template name="right-page">
<tr valign="top">
<!-- the rest of text blocks --> 
<xsl:apply-templates select="block[preceding-sibling::block]"/>
<!-- link to the next section and copyright info --> 
<xsl:call-template name="rightside"/>
</tr>
</xsl:template>

<!-- intro for a section page -->
<xsl:template match="intro">
<td><table cellspacing="0" cellpadding="0" border="0">
<tr><td><img alt="" src="img/e.gif" width="1" height="326"/></td></tr>
<tr valign="top">
<xsl:apply-templates select="column"/>
</tr></table></td>
</xsl:template>

<!-- textual blocks for a section page -->
<xsl:template match="block">
<td><table cellspacing="0" cellpadding="0" border="0">
<tr><td><img alt="" src="img/e.gif" width="1" height="18"/></td></tr>
<tr valign="top">
<td><img alt="" src="img/e.gif" width="1" height="30"/></td>
<!-- top heading of the block -->
<xsl:apply-templates select="name"/>
</tr>
<tr valign="top">
<td><img alt="" src="img/e.gif" width="1" height="278"/></td>
<!-- thumb images for each column (if any) -->
<xsl:apply-templates mode="image" select="column"/>
</tr>
<tr valign="top">
<td></td>
<!-- text of each column -->
<xsl:apply-templates select="column"/>
</tr></table></td>
</xsl:template>

<!-- top heading of the block -->
<xsl:template match="block/name">
<!-- we generate the name of the image for the top heading from its text -->
<xsl:variable name="t"><xsl:apply-templates/></xsl:variable>
<xsl:variable name="src" select="concat($s-path, 'img/h-', $t)"/>
<xsl:if test="$gen='yes'">
  <xsl:message>generate: <xsl:value-of select="$src"/></xsl:message>
<!-- we create an svg file from the heading. 
     the size is 4 times larger for better anti-aliasing. -->
  <xsl:document href="{$src}.svg" encoding="utf-8" method="xml" omit-xml-declaration="yes" indent="yes">
    <svg width="1600px" height="200px">
      <rect width="1800px" height="400px" x="-100px" y="-100px" style="fill: {$dark}"/>
      <text style="fill: #ffffff; font-family: @{$font-bold}; font-size: {$top-fs * 4}pt;" x="100px" y="100px"><xsl:value-of select="."/></text>
    </svg><xsl:text>
    </xsl:text>
  </xsl:document>
<!-- we convert svg image to png format, scale it 4 times down and
     convert to gif with 256 colors. 
     the background red color is made transparent.
     then we convert it to 16 colors. -->                                                      
  <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert ', $src, '.svg ', $src, '.png'))"/></xsl:message>               
  <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify +dither -filter Triangle -geometry 25%x25% ', $src, '.png'))"/></xsl:message>               
  <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert +dither ', $src, '.png ', $src, '.gif'))"/></xsl:message>               
  <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify +dither -fill ', $dark, ' -transparent ', $dark, ' ', $src, '.gif'))"/></xsl:message>               
  <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -colors 16 ', $src, '.gif'))"/></xsl:message>               
  <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -crop 0x0 ', $src, '.gif'))"/></xsl:message>               
<!-- we delete temporary files-->    
  <xsl:message><xsl:value-of select="dk-r:delete(concat($src, '.png'))"/></xsl:message>               
  <xsl:message><xsl:value-of select="dk-r:delete(concat($src, '.svg'))"/></xsl:message>               
</xsl:if>

<td width="{$w-btw-block - 1}" align="right">
  <img src="img/ar-dark.gif" width="{dk-g:getw('img/ar-dark.gif')}" height="{dk-g:geth('img/ar-dark.gif')}" border="0">
    <xsl:call-template name="scroll"/>
  </img><img alt="" src="img/e.gif" width="{$w-btw-arrow}" height="1"/>
</td>
<!-- each block may contain one or more columns. Top heading is places over 
     all the columns of the corresponding block -->  
<td colspan="{count(following-sibling::column) * 2 - 1}">
  <p style="margin: 0px">
    <xsl:choose>
      <xsl:when test="@href">
        <a href="{@href}">
          <xsl:call-template name="target"/>
          <em><img alt="{.}" src="img/h-{$t}.gif" width="{dk-g:getw(concat($src, '.gif'))}" height="{dk-g:geth(concat($src, '.gif'))}" border="0"/></em>
        </a>
      </xsl:when>
      <xsl:otherwise>
        <em><img alt="{.}" src="img/h-{$t}.gif" width="{dk-g:getw(concat($src, '.gif'))}" height="{dk-g:geth(concat($src, '.gif'))}"/></em>
      </xsl:otherwise>
    </xsl:choose>
  </p>
</td>
</xsl:template>  

<!-- each column may contain a thumb image -->     
<xsl:template mode="image" match="column">
<td colspan="2">
<!-- it contains a thumb image if there is an id attribute --> 
  <xsl:if test="@id">
    <xsl:if test="$gen='yes'">
      <xsl:call-template name="thumb"/>
    </xsl:if>
<!-- vertical position should differ from that of the preceding thumb;
     for the first block, we have to avoid the very bottom position 
     because the studio name is placed there -->
    <xsl:variable name="pos">
      <xsl:choose>
        <xsl:when test="parent::block[not(preceding-sibling::block)]">
          <xsl:value-of select="dk-u:getRandom(1, 4, $prev)"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="dk-u:getRandom(1, 5, $prev)"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:message>vertical position: <xsl:value-of select="$pos"/></xsl:message>
    <saxon:assign name="prev" select="$pos"/>            
    <img alt="" src="img/e.gif" width="1" height="{5 + (49 * ($pos - 1))}"/><br/>
    <xsl:if test="not(preceding-sibling::column)">
      <img alt="" src="img/e.gif" width="{$w-btw-block - $w-btw-col - 1}" height="1"/>
    </xsl:if>

<!-- if the thumb is placed on the very bottom we have to use an image
     with two-colored background; other thumbs have the dark background only -->
    <xsl:variable name="image">
      <xsl:choose>
        <xsl:when test="$pos = 4"><img alt="[click to view]" src="img/i-{@id}-2.gif" width="128" height="126" border="0"/></xsl:when>
        <xsl:otherwise><img alt="[click to view]" src="img/i-{@id}-1.gif" width="128" height="126" border="0"/></xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

<!-- thumb may be either linked to a separate display page or url,
     or not linked at all --> 
    <xsl:choose>
      <xsl:when test="@display">
<!-- linked to a separate display page -->
        <a href="{@display}.html">
          <xsl:call-template name="target">
            <xsl:with-param name="target" select="1"/>
          </xsl:call-template>
          <xsl:copy-of select="$image"/>
        </a>
        <xsl:call-template name="display"/>
      </xsl:when>
      <xsl:when test="@href">
<!-- linked to a url -->
        <a href="{@href}">
          <xsl:call-template name="target"/>
          <xsl:copy-of select="$image"/>
        </a>
      </xsl:when>
      <xsl:when test="preceding-sibling::name/@href">
        <a href="{preceding-sibling::name/@href}">
          <xsl:call-template name="target"/>
          <xsl:copy-of select="$image"/>
        </a>
      </xsl:when>
      <xsl:otherwise>
<!-- not linked -->
        <xsl:copy-of select="$image"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:if>
</td>
</xsl:template> 

<xsl:template name="rightside">
<td><img alt="" src="img/e.gif" width="{$w-btw-block - dk-g:getw('img/ar-dark.gif') - $w-btw-arrow}" height="1"/></td>
<td><table cellspacing="0" cellpadding="0" border="0">
<tr><td><img alt="" src="img/e.gif" width="1" height="18"/></td></tr>
<!-- arrow and small textual button linked to the next section 
     (excluding two sections after "testimonials" linked to separate urls) or
     to the first section if the current section is the last one -->
<tr valign="top">
<td align="right"><a>
  <xsl:attribute name="href">
    <xsl:choose>
      <xsl:when test="$current='archive'">
        <xsl:value-of select="document($site)//subsection[1]/@url"/>
      </xsl:when>
      <xsl:when test="$current='testimonials'">
        <xsl:value-of select="document($site)//subsection[@url='source.html']/@url"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="document($site)//subsection[preceding::subsection[1][substring-before(@url, '.html') = $current]]/@url"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:attribute> 
<img alt="" src="img/ar-dark.gif" width="{dk-g:getw('img/ar-dark.gif')}" height="{dk-g:geth('img/ar-dark.gif')}" border="0"/></a></td>
<td rowspan="3"><img alt="" src="img/e.gif" width="{$w-btw-arrow}" height="1" border="0"/></td>
<td><a>
      <xsl:attribute name="href">
        <xsl:choose>
          <xsl:when test="$current='archive'">
            <xsl:value-of select="document($site)//subsection[1]/@url"/>
          </xsl:when>
          <xsl:when test="$current='testimonials'">
            <xsl:value-of select="document($site)//subsection[@url='source.html']/@url"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="document($site)//subsection[preceding::subsection[1][substring-before(@url, '.html') = $current]]/@url"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <img border="0">
        <xsl:choose>
          <xsl:when test="$current='archive'">
            <xsl:attribute name="alt">Start over: <xsl:value-of select="document($site)//subsection[1]"/></xsl:attribute>
            <xsl:attribute name="src">img/l-startover.gif</xsl:attribute>
            <xsl:attribute name="width"><xsl:value-of select="dk-g:getw('img/l-startover.gif')"/></xsl:attribute>
            <xsl:attribute name="height"><xsl:value-of select="dk-g:geth('img/l-startover.gif')"/></xsl:attribute>
          </xsl:when>
          <xsl:when test="$current='testimonials'">
            <xsl:attribute name="alt">Next: <xsl:value-of select="document($site)//subsection[@url='source.html']"/></xsl:attribute>
            <xsl:attribute name="src">img/l-next.gif</xsl:attribute>
            <xsl:attribute name="width"><xsl:value-of select="dk-g:getw('img/l-next.gif')"/></xsl:attribute>
            <xsl:attribute name="height"><xsl:value-of select="dk-g:geth('img/l-next.gif')"/></xsl:attribute>
          </xsl:when>
          <xsl:otherwise>
            <xsl:attribute name="alt">Next: <xsl:value-of select="document($site)//subsection[preceding::subsection[1][substring-before(@url, '.html') = $current]]"/></xsl:attribute>
            <xsl:attribute name="src">img/l-next.gif</xsl:attribute>
            <xsl:attribute name="width"><xsl:value-of select="dk-g:getw('img/l-next.gif')"/></xsl:attribute>
            <xsl:attribute name="height"><xsl:value-of select="dk-g:geth('img/l-next.gif')"/></xsl:attribute>
          </xsl:otherwise>
        </xsl:choose></img>
</a></td>
</tr>
<tr valign="top"><td><img alt="" src="img/e.gif" width="1" height="284"/></td></tr>
<tr valign="top">
<!-- arrow and small textual mailto: link -->
<td><img alt="" src="img/e.gif" width="1" height="6"/><br/>
<a href="{document($site)//button[text() = 'Write Us']/@url}"><img alt="" src="img/ab-light.gif" width="{dk-g:getw('img/at-light.gif')}" height="{dk-g:geth('img/at-light.gif')}" border="0"/></a></td>
<td class="copy"><a href="{document($site)//button[text() = 'Write Us']/@url}"><img alt="Write Us" src="img/l-writeus.gif" width="{dk-g:getw('img/l-writeus.gif')}" height="{dk-g:geth('img/l-writeus.gif')}" border="0"/></a><br/>
<img alt="" src="img/e.gif" width="1" height="10"/><br/>
<!-- copyright info -->
<xsl:apply-templates select="document($site)//copyright"/></td>
</tr></table></td>
<td><img alt="" src="img/e.gif" width="10" height="1"/></td>
</xsl:template> 

<xsl:template match="block/name/text()|buttons/button/text()|topimg/@say/text()">
<!-- we generate names for texual images from the corresponding texts
     using recursion -->
<xsl:call-template name="rec-name">
  <xsl:with-param name="s" select="normalize-space(.)"/>
</xsl:call-template>
</xsl:template>

<xsl:template name="rec-name">
<xsl:param name="s"/>
<!-- we recurcively remove symbols other than letters and numbers
     from the corresponding text and crop too long names to get
     the name for the corresponding image -->
<xsl:choose>
  <xsl:when test="contains($s, ' ')">
    <xsl:call-template name="rec-name">
      <xsl:with-param name="s" select="concat(substring-before($s, ' '), substring-after($s, ' '))"/>
    </xsl:call-template>
  </xsl:when>
  <xsl:when test="contains($s, '!')">
    <xsl:call-template name="rec-name">
      <xsl:with-param name="s" select="concat(substring-before($s, '!'), substring-after($s, '!'))"/>
    </xsl:call-template>
  </xsl:when>
  <xsl:when test="contains($s, '?')">
    <xsl:call-template name="rec-name">
      <xsl:with-param name="s" select="concat(substring-before($s, '?'), substring-after($s, '?'))"/>
    </xsl:call-template>
  </xsl:when>
  <xsl:when test="contains($s, '.')">
    <xsl:call-template name="rec-name">
      <xsl:with-param name="s" select="concat(substring-before($s, '.'), substring-after($s, '.'))"/>
    </xsl:call-template>
  </xsl:when>
  <xsl:when test="contains($s, ',')">
    <xsl:call-template name="rec-name">
      <xsl:with-param name="s" select="concat(substring-before($s, ','), substring-after($s, ','))"/>
    </xsl:call-template>
  </xsl:when>
  <xsl:when test="contains($s, $quote)">
    <xsl:call-template name="rec-name">
      <xsl:with-param name="s" select="concat(substring-before($s, $quote), substring-after($s, $quote))"/>
    </xsl:call-template>
  </xsl:when>
  <xsl:when test="contains($s, $apostrophe)">
    <xsl:call-template name="rec-name">
      <xsl:with-param name="s" select="concat(substring-before($s, $apostrophe), substring-after($s, $apostrophe))"/>
    </xsl:call-template>
  </xsl:when>
  <xsl:when test="contains($s, '%')">
    <xsl:call-template name="rec-name">
      <xsl:with-param name="s" select="concat(substring-before($s, '.'), substring-after($s, '.'))"/>
    </xsl:call-template>
  </xsl:when>
  <xsl:when test="string-length($s) &gt; 14">
    <xsl:call-template name="rec-name">
      <xsl:with-param name="s" select="substring($s, 1, 14)"/>
    </xsl:call-template>
  </xsl:when>
  <xsl:otherwise>
    <xsl:value-of select="translate($s, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ', 'abcdefghijklmnopqrstuvwxyz')"/>
  </xsl:otherwise>
</xsl:choose>
</xsl:template>

<xsl:template name="thumb">
<!-- to generate thumb images, export thumbs/{@id}.png, full color + alpha -->
<xsl:variable name="source" select="concat($s-path, 'thumbs/', @id, '.png')"/>
<xsl:variable name="src" select="concat($s-path, 'img/i-', @id)"/>        
<xsl:choose>
  <xsl:when test="not(dk-r:exists($source))">
    <xsl:message>no such source file: <xsl:value-of select="$source"/> !</xsl:message>
  </xsl:when>
  <xsl:otherwise>
    <xsl:message>generate: <xsl:value-of select="$src"/></xsl:message>
<!-- generating new thumbs from the given png image and four shaded background images --> 
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'composite ', $source, ' img/bg1.gif +dither -geometry +51+48 ', $src, '-1.gif'))"/></xsl:message>               
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify +dither -transparent ', $dark, ' ', $src, '-1.gif'))"/></xsl:message>               
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -crop 0x0 ', $src, '-1.gif'))"/></xsl:message>               
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert ', $src, '-1.gif ', $src, '-1.png'))"/></xsl:message>               
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert -colors 64 +dither ', $src, '-1.png ', $src, '-1.gif'))"/></xsl:message>               
 
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'composite ', $source , ' img/bg2.gif +dither -geometry +51+48 ', $src, '-2.gif'))"/></xsl:message>               
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -fill ', $dark, '  -opaque ', $light, ' +dither ', $src, '-2.gif'))"/></xsl:message>               
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify +dither -colors 128 ', $src, '-2.gif'))"/></xsl:message>               
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify +dither -transparent ', $dark, ' ', $src, '-2.gif'))"/></xsl:message>               
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -crop 0x0 ', $src, '-2.gif'))"/></xsl:message>               
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -colors 64 +dither ', $src, '-2.gif'))"/></xsl:message>               
    
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'composite ', $source, ' img/bg3.gif +dither -geometry +51+48 ', $src, '-3.gif'))"/></xsl:message>               
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -fill ', $dark, '  -opaque ', $light, ' +dither ', $src, '-3.gif'))"/></xsl:message>               
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify +dither -colors 128 ', $src, '-3.gif'))"/></xsl:message>               
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify +dither -transparent ', $dark, ' ', $src, '-3.gif'))"/></xsl:message>               
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -crop 0x0 ', $src, '-3.gif'))"/></xsl:message>               
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -colors 64 +dither ', $src, '-3.gif'))"/></xsl:message>               
   
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'composite ', $source, ' img/bg4.gif +dither -geometry +51+48 ', $src, '-4.gif'))"/></xsl:message>            
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'composite ', $source, ' img/bg4.gif +dither -geometry +51+48 ', $src, '-4.gif'))"/></xsl:message>                  
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify +dither -transparent ', $light, ' ', $src, '-4.gif'))"/></xsl:message>               
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -crop 0x0 ', $src, '-4.gif'))"/></xsl:message>               
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert ', $src, '-4.gif ', $src, '-4.png'))"/></xsl:message>               
    <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert -colors 64 +dither ', $src, '-4.png ', $src, '-4.gif'))"/></xsl:message>               
    
    <xsl:message><xsl:value-of select="dk-r:delete(concat($src, '-2-b.gif'))"/></xsl:message>               
    <xsl:message><xsl:value-of select="dk-r:delete(concat($src, '-3-b.gif'))"/></xsl:message>               
    <xsl:message><xsl:value-of select="dk-r:delete(concat($src, '-1.png'))"/></xsl:message>               
    <xsl:message><xsl:value-of select="dk-r:delete(concat($src, '-4.png'))"/></xsl:message>               
  </xsl:otherwise>
</xsl:choose>
</xsl:template> 

<!-- generating a separate html page displaying some logo or paper stuff -->
<xsl:template name="display">
<xsl:message>display: <xsl:value-of select="@display"/></xsl:message>
<xsl:variable name="src" select="concat($s-path, 'display/', @display)"/>             
<xsl:if test="not(dk-r:exists(concat($src, '.png')) or dk-r:exists(concat($src, '.jpg')) or dk-r:exists(concat($src, '.gif')))">
  <xsl:message>no such source file: <xsl:value-of select="$src"/>!</xsl:message>
</xsl:if>
<xsl:variable name="fullsrc">
  <xsl:choose>
    <xsl:when test="dk-r:exists(concat($src, '.png'))">
      <xsl:value-of select="concat($s-path, 'display/', @display, '.png')"/>          
    </xsl:when>
    <xsl:when test="dk-r:exists(concat($src, '.jpg'))">
      <xsl:value-of select="concat($s-path, 'display/', @display, '.jpg')"/>          
    </xsl:when>
    <xsl:when test="dk-r:exists(concat($src, '.gif'))">
      <xsl:value-of select="concat($s-path, 'display/', @display, '.gif')"/>      
    </xsl:when>
  </xsl:choose>
</xsl:variable>
<xsl:document href="{@display}.html" encoding="utf-8" method="html" omit-xml-declaration="no" indent="no">
<xsl:call-template name="credits"/>
<html>
<head>
  <title><xsl:value-of select="preceding-sibling::name"/> - <xsl:value-of select="preceding::title"/> - Dmitry Kirsanov Studio</title>
</head>
<body onload="window.focus()">
  <xsl:attribute name="bgcolor">
    <xsl:choose>
      <xsl:when test="$current = 'paper'">#b2b2b2</xsl:when>
      <xsl:otherwise>#ffffff</xsl:otherwise>
    </xsl:choose>
  </xsl:attribute>
<table width="100%" height="100%" cellspacing="0" cellpadding="0" border="0"><tr>
<td align="center" valign="center"><a href="javascript:window.close()"><img alt="{preceding-sibling::name} image, click to return" src="{substring-after($fullsrc, $s-path)}" border="0"/></a></td>
</tr></table>
</body>
</html>
</xsl:document>
</xsl:template> 

<!-- text column of a block -->
<xsl:template match="column">
<xsl:choose>
  <xsl:when test="not(preceding-sibling::column) and not(ancestor::intro)">
    <td width="{$w-btw-block}" align="right"><img alt="" src="img/e.gif" width="{$w-btw-block}" height="6"/><br/><img src="img/ar-light.gif" width="{dk-g:getw('img/ar-light.gif')}" height="{dk-g:geth('img/ar-light.gif')}" border="0"><xsl:call-template name="scroll"/></img><img alt="" src="img/e.gif" width="{$w-btw-arrow}" height="1"/></td>
  </xsl:when>
  <xsl:when test="preceding-sibling::column">
    <td width="{$w-btw-col}" align="right"><img alt="" src="img/e.gif" width="{$w-btw-col}" height="1"/></td>
  </xsl:when>
</xsl:choose>
<td width="{$w-col}">
  <p class="ctext">
    <xsl:apply-templates/><br/>   
    <img alt="" src="img/e.gif" width="{$w-col}" height="6"/>
  </p>
</td>   
</xsl:template> 

<xsl:template match="a">
<a><xsl:copy-of select="@*"/><xsl:call-template name="target"/><xsl:apply-templates/></a>
</xsl:template>

<xsl:template match="br"><br/></xsl:template>

<xsl:template match="i">
<xsl:copy><xsl:apply-templates/></xsl:copy>
</xsl:template>

<xsl:template match="colors/color">
<xsl:variable name="t" select="@id"/>            
<!-- we generate some shared images for each of specified colors -->

<!-- generate right arrow -->
<xsl:variable name="src-r" select="concat($s-path, 'img/ar-', $t)"/>            
<xsl:message>generate: <xsl:value-of select="$src-r"/></xsl:message>

<xsl:document href="{$src-r}.svg" method="xml" encoding="utf-8" omit-xml-declaration="yes" indent="yes">
  <svg width="400px" height="400px">
    <rect width="600px" height="600px" x="-100px" y="-100px" style="fill: {text()}"/>
    <circle cx="100" cy="100" r="16" fill="{ancestor::colors/color[@id != current()/@id]/text()}"/>
    <circle cx="128" cy="128" r="16" fill="{ancestor::colors/color[@id != current()/@id]/text()}"/>
    <circle cx="100" cy="156" r="16" fill="{ancestor::colors/color[@id != current()/@id]/text()}"/>
  </svg><xsl:text>
  </xsl:text>
</xsl:document> 
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert ', $src-r, '.svg ', $src-r, '.png'))"/></xsl:message>
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify +dither -filter Box -geometry 25%x25% ', $src-r, '.png'))"/></xsl:message>
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert +dither ', $src-r, '.png ', $src-r, '.gif'))"/></xsl:message>
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify +dither -fill ', text(), ' -transparent ', text(), ' ', $src-r, '.gif'))"/></xsl:message>
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -colors 16 ', $src-r, '.gif'))"/></xsl:message>
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -crop 0x0 ', $src-r, '.gif'))"/></xsl:message>
<xsl:message><xsl:value-of select="dk-r:delete(concat($src-r, '.png'))"/></xsl:message>
<xsl:message><xsl:value-of select="dk-r:delete(concat($src-r, '.svg'))"/></xsl:message> 

<!-- generate top arrow -->
<xsl:variable name="src-t" select="concat($s-path, 'img/at-', $t)"/>            
<xsl:message>generate: <xsl:value-of select="$src-t"/></xsl:message>

<xsl:document href="{$src-t}.svg" method="xml" encoding="utf-8" omit-xml-declaration="yes" indent="yes">
  <svg width="400px" height="400px">
    <rect width="600px" height="600px" x="-100px" y="-100px" style="fill: {text()}"/>
    <circle cx="128" cy="100" r="16" fill="{ancestor::colors/color[@id != current()/@id]/text()}"/>
    <circle cx="100" cy="128" r="16" fill="{ancestor::colors/color[@id != current()/@id]/text()}"/>
    <circle cx="156" cy="128" r="16" fill="{ancestor::colors/color[@id != current()/@id]/text()}"/>
  </svg><xsl:text>
  </xsl:text>
</xsl:document>
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert ', $src-t, '.svg ', $src-t, '.png'))"/></xsl:message>
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify +dither -filter Box -geometry 25%x25% ', $src-t, '.png'))"/></xsl:message>
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert +dither ', $src-t, '.png ', $src-t, '.gif'))"/></xsl:message>
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify +dither -fill ', text(), ' -transparent ', text(), ' ', $src-t, '.gif'))"/></xsl:message>
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -colors 16 ', $src-t, '.gif'))"/></xsl:message>
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -crop 0x0 ', $src-t, '.gif'))"/></xsl:message>
<xsl:message><xsl:value-of select="dk-r:delete(concat($src-t, '.png'))"/></xsl:message>   
<xsl:message><xsl:value-of select="dk-r:delete(concat($src-t, '.svg'))"/></xsl:message>   

<!-- generate bottom arrow -->
<xsl:variable name="src-b" select="concat($s-path, 'img/ab-', $t)"/>            
<xsl:message>generate: <xsl:value-of select="$src-b"/></xsl:message>

<xsl:document href="{$src-b}.svg" method="xml" encoding="utf-8" omit-xml-declaration="yes" indent="yes">
  <svg width="400px" height="400px">
    <rect width="600px" height="600px" x="-100px" y="-100px" style="fill: {text()}"/>
    <circle cx="100" cy="100" r="16" fill="{ancestor::colors/color[@id != current()/@id]/text()}"/>
    <circle cx="156" cy="100" r="16" fill="{ancestor::colors/color[@id != current()/@id]/text()}"/>
    <circle cx="128" cy="128" r="16" fill="{ancestor::colors/color[@id != current()/@id]/text()}"/>
  </svg><xsl:text>
  </xsl:text>
</xsl:document>
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert ', $src-b, '.svg ', $src-b, '.png'))"/></xsl:message>       
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify +dither -filter Box -geometry 25%x25% ', $src-b, '.png'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert +dither ', $src-b, '.png ', $src-b, '.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify +dither -fill ', text(), ' -transparent ', text(), ' ', $src-b, '.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -colors 16 ', $src-b, '.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -crop 0x0 ', $src-b, '.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:delete(concat($src-b, '.png'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:delete(concat($src-b, '.svg'))"/></xsl:message>               

<!-- generate mono-colored bg -->
<xsl:variable name="src-m" select="concat($s-path, 'img/b-', $t)"/>        
<xsl:message>generate: <xsl:value-of select="$src-m"/></xsl:message>

<xsl:document href="{$src-m}.svg" method="xml" encoding="utf-8" omit-xml-declaration="yes" indent="yes">
  <svg width="273px" height="209px">
    <rect width="273px" height="209px" x="0px" y="0px" style="fill: {text()}"/>
  </svg><xsl:text>
  </xsl:text>
</xsl:document>
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert ', $src-m, '.svg ', $src-m, '.png'))"/></xsl:message>               <xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert +dither ', $src-m, '.png ', $src-m, '.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:delete(concat($src-m, '.svg'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:delete(concat($src-m, '.png'))"/></xsl:message>               

<!-- generate two-colored bg -->
<xsl:variable name="src-tc" select="concat($s-path, 'img/b-', $t, '-', ancestor::colors/color[@id != current()/@id]/@id)"/>            
<xsl:message>generate: <xsl:value-of select="$src-tc"/></xsl:message>

<xsl:document href="{$src-tc}.svg" encoding="utf-8" method="xml" omit-xml-declaration="yes" indent="yes">
  <svg width="273px" height="209px">
    <xsl:choose>
      <xsl:when test="$t = 'dark'">
        <rect width="273px" height="135px" x="0px" y="0px" style="fill: {text()}"/>
        <rect width="273px" height="74px" x="0px" y="135px" style="fill: {ancestor::colors/color[@id != current()/@id]/text()}"/>
      </xsl:when>
      <xsl:otherwise>
        <rect width="273px" height="72px" x="0px" y="0px" style="fill: {ancestor::colors/color[@id != current()/@id]/text()}"/>
        <rect width="273px" height="137px" x="0px" y="72px" style="fill: {text()}"/>
      </xsl:otherwise>
    </xsl:choose>
  </svg><xsl:text>
  </xsl:text>
</xsl:document>
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert ', $src-tc, '.svg ', $src-tc, '.png'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert +dither ', $src-tc, '.png ', $src-tc, '.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:delete(concat($src-tc, '.svg'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:delete(concat($src-tc, '.png'))"/></xsl:message>               
</xsl:template> 

<xsl:template match="buttons/button">
<!-- generate button -->
<xsl:variable name="t"><xsl:apply-templates/></xsl:variable>            
<xsl:variable name="src" select="concat($s-path, 'img/l-', $t)"/>            
<xsl:message>generate: <xsl:value-of select="$src"/></xsl:message>

<xsl:document href="{$src}.svg" encoding="utf-8" method="xml" omit-xml-declaration="yes" indent="yes">
  <svg width="400px" height="400px">
    <rect width="600px" height="600px" x="-100px" y="-100px" style="fill: {//color[@id = current()/@bg]/text()}"/>
    <text style="fill: #ffffff; font-family: @{$font-button}; font-size: {$but-fs * 4}pt;" x="100px" y="100px"><xsl:value-of select="translate(text(), 'abcdefghijklmnopqrstuvwxyz', 'ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/></text>
  </svg><xsl:text>
  </xsl:text>
</xsl:document>
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert ', $src, '.svg ', $src, '.png'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify +dither -filter Triangle -geometry 25%x25% ', $src, '.png'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert +dither ', $src, '.png ', $src, '.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify +dither -fill ', //color[@id = current()/@bg]/text(), ' -transparent ', //color[@id = current()/@bg]/text(), ' ', $src, '.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -colors 16 ', $src, '.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -crop 0x0+0+7 ', $src, '.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:delete(concat($src, '.png'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:delete(concat($src, '.svg'))"/></xsl:message>               
</xsl:template> 

<xsl:template name="make-bg">
<xsl:message>make-bg</xsl:message>
<!-- source: bw.png: black square on white, no transparency -->
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert img/bw.png img/bw-b.png'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -blur 31x10 img/bw-b.png'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -colors 2 -dither img/bw-b.png'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'convert +dither img/bw-b.png img/bw-b.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -transparent #f5f5f5 img/bw-b.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -fill #000000 -opaque #272727 img/bw-b.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'composite +dither -dissolve ', $dissolve, ' img/bw-b.gif img/b-dark.gif img/bg1.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -fuzz 1000 -fill ', $dark, ' -opaque ', $dark, ' img/bg1.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'composite +dither -dissolve ', $dissolve, ' img/bw-b.gif img/b-dark-light.gif img/bg2.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -fuzz 1000 -fill ', $light, ' -opaque ', $light, ' img/bg2.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -fuzz 1000 -fill ', $dark, ' -opaque ', $dark, ' img/bg2.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'composite +dither -dissolve ', $dissolve, ' img/bw-b.gif img/b-light-dark.gif img/bg3.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -fuzz 1000 -fill ', $dark, ' -opaque ', $dark, ' img/bg3.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -fuzz 1000 -fill ', $light, ' -opaque ', $light, ' img/bg3.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'composite +dither -dissolve ', $dissolve, ' img/bw-b.gif img/b-light.gif img/bg4.gif'))"/></xsl:message>               
<xsl:message><xsl:value-of select="dk-r:run(concat($im-path, 'mogrify -fuzz 1000 -fill ', $light , ' -opaque ', $light, ' img/bg4.gif'))"/></xsl:message>               
</xsl:template>

<!-- target for links: the same window for inner textual links; 
to the second browser's window for displaying materials and other websites -->

<xsl:template name="target">
<xsl:param name="target">0</xsl:param>
<xsl:if test="contains(@href, '/') or $target=1 or (not(@href) and contains(preceding-sibling::name/@href, '/'))">
  <xsl:attribute name="target">illustration</xsl:attribute>
  <xsl:attribute name="onclick">popup()</xsl:attribute>
</xsl:if>
</xsl:template>

<!-- credits: -->

<xsl:template name="credits">
<xsl:message></xsl:message>
<xsl:message>Compiling the predictablyperfect.com site</xsl:message>
<xsl:message>Design: Dmitry Kirsanov Studio, www.kirsanov.com</xsl:message> 
<xsl:message>XSLT coding: Alina Kirsanova</xsl:message>
<xsl:message><xsl:value-of select="$current"/></xsl:message>
<xsl:message></xsl:message>
<xsl:comment>
Design: Dmitry Kirsanov Studio, http://www.kirsanov.com
HTML and XSLT: Alina Kirsanova
Java and Javascript: Ilya Oussov
</xsl:comment>
</xsl:template>

<!-- css: -->

<xsl:template name="css">
<link rel="stylesheet" href="site.css"/>
<xsl:if test="$current='index'">
  <xsl:message>site.css</xsl:message>
  <xsl:document href="site.css" method="text" omit-xml-declaration="yes">empty {}
body    {font-family: Georgia, Times, serif;}
a       {text-decoration: none; font-family: Arial, sans serif; font-weight: bold; font-style: normal;}
p       {font-size: x-small; line-height: 140%;}
.copy   {font-size: xx-small; line-height: 150%;}
p.ctext {font-size: small; line-height: 150%; color: #000000;}
  </xsl:document>
  <style type="text/css">
    <xsl:apply-templates mode="style" select="//frame"/>
  </style>
</xsl:if>
</xsl:template>

<!-- javascript: --> 

<!-- Here, animation functions are defined that, at random time
intervals, change the frames in the front page thumbnails.  Note
that all animated images must contain the same number of frames;
here, the first image is used to get the count of frames.

Also, functions and event handlers for horizontal scrolling by
"vertical" keys (home, end, up/down arrows, pgup/pgdn) and mouse
wheel are defined here. Mousewheeling horizontally currently only
works in MSIE 6. Mozilla bug: end key does not work (scrolls to the
end and then back again). MSIE provision: left/right arrow keys are
also redefined because their default scroll distance is too small
for comfortable horizonal scrolling. -->

<!--//additional checks if needed:
NS4=(document.layers)?true:false;ver4=(NS4||IE);NS4old=(NS4&amp;&amp;(parseFloat(navigator.appVersion)&lt;4.02));IE5=IE&amp;&amp;DOM;IEW=IE&amp;&amp;!isMac;IE4W=IE4&amp;&amp;IEW;IE5W=IE5&amp;&amp;IEW;IE4M=IE4&amp;&amp;isMac;NS=navigator.appName==("Netscape");
-->

<xsl:template name="script">
<script language="JavaScript" type="text/javascript"><xsl:text>
</xsl:text><xsl:comment><xsl:text>
</xsl:text><xsl:if test="$current='index'">a=new Array(4);function r(i,n){if(DOM){<xsl:apply-templates mode="script" select="image[1]/frame"/>if (++n==<xsl:value-of select="count(image[1]/frame) + 1"/>)n=1;document.getElementById("i"+i+"-"+n).style.visibility="visible";ff=10*i+n;a[i]=setTimeout("cr("+ff+")",Math.random()*3200+1200);}}function cr(ff){r(Math.floor(ff/10),(ff%10));}function s(i){clearTimeout(a[i]);}</xsl:if>function popup(){newwin = window.open('','illustration');newwin.focus()}function hscr(){window.scrollBy(-event.wheelDelta/3,0);}document.onmousewheel=hscr;NS6=(navigator.vendor==("Netscape6")||navigator.product==("Gecko"));IE=(document.all)?true:false;DOM=(document.getElementById)?true:false;IE4=IE&amp;&amp;!DOM;IE5=(IE&amp;&amp;navigator.appVersion.indexOf("5.")!=-1);isMac=(navigator.appVersion.indexOf("Mac")!=-1);IE5M=IE5&amp;&amp;isMac;if(IE5M){DOM=false;IE4=true};<xsl:if test="$current!='index'">function cursorHand(obj){if(IE)obj.style.cursor='hand';if(NS6)obj.style.cursor='pointer';window.status="click to scroll";};function ks(c){switch(c){case 33:window.scrollBy(-750,0);break;case 34:window.scrollBy(750,0);break;case 38:window.scrollBy(-50,0);break;case 40:window.scrollBy(50,0);break;case 37:window.scrollBy(-40,0);break;case 39:window.scrollBy(40,0);break;case 35:window.scrollBy(10000,0);break;case 36:window.scrollBy(-10000,0);break;}}function k(e){ks(event.keyCode);}function km(event){ks(event.keyCode);};if(NS6)document.onkeypress=km;else document.onkeydown=k;</xsl:if><xsl:text>
</xsl:text>//</xsl:comment><xsl:text>
</xsl:text></script>
</xsl:template>

<!-- plugin detection for IE -->
<xsl:template name="detect-ie">Private i,x,MM_FlashControlVersion
On Error Resume Next
x=null
MM_FlashControlVersion=0
var Flashmode
FlashMode=False
For i=5 To 1 Step -1
Set x=CreateObject("ShockwaveFlash.ShockwaveFlash."&amp;i)
MM_FlashControlInstalled=IsObject(x)
If MM_FlashControlInstalled Then
MM_FlashControlVersion=CStr(i)
Exit For
End If
Next
FlashMode=(MM_FlashControlVersion&gt;=3)</xsl:template>

<!-- plugin detection for NN -->
<xsl:template name="detect-nn">{if(navigator.plugins["Shockwave Flash"]){var plugin_version=0;var words=navigator.plugins["Shockwave Flash"].description.split(" ");for(var i=0;i&lt;words.length;++i){if(isNaN(parseInt(words[i])))continue;plugin_version=words[i];}if(plugin_version&gt;=3){var plugin=navigator.plugins["Shockwave Flash"];var numTypes=plugin.length;for(j=0;j&lt;numTypes;j++){mimetype=plugin[j];if(mimetype){if(mimetype.enabledPlugin&amp;&amp;(mimetype.suffixes.indexOf("swf")!=-1))FlashMode=1;if(navigator.mimeTypes["application/x-shockwave-flash"]==null)FlashMode=0;}}}}}</xsl:template>

<!-- flash for main page: -->
<xsl:template name="flash">
<script language="VBScript" type="text/vbscript"><xsl:text>
</xsl:text><xsl:comment><xsl:text>
</xsl:text><xsl:call-template name="detect-ie"/>
If FlashMode=True Then
document.write "&lt;object classid=""clsid:D27CDB6E-AE6D-11cf-96B8-444553540000"" codebase=""http://active.macromedia.com/flash2/cabs/swflash.cab#version=2,0,0,0"" width=""185"" height=""178""&gt;&lt;param name=""movie"" value=""menu/menu.swf""&gt;&lt;param name=""quality"" value=""high""&gt;&lt;param name=""loop"" value=""false""&gt;&lt;param name=""bgcolor"" value=""<xsl:value-of select="$light"/>""&gt;&lt;param name=""play"" value=""false""&gt;&lt;" &amp; "/object&gt;"
Else
document.write "&lt;img alt="""" src=""menu/menu.gif"" usemap=""#menu"" border=""0""  width=""185"" height=""178""&gt;"
End If<xsl:text>
</xsl:text></xsl:comment><xsl:text>
</xsl:text></script>
<script language="JavaScript" type="text/javascript"><xsl:text>
</xsl:text><xsl:comment><xsl:text>
</xsl:text>if(navigator.appName=="Netscape"||isMac){FlashMode=0;if(navigator.plugins&amp;&amp;navigator.plugins.length&gt;0)<xsl:call-template name="detect-nn"/>if(FlashMode==1){document.write('&lt;embed src="menu/menu.swf" quality="best" bgcolor="<xsl:value-of select="$light"/>" WIDTH="185" HEIGHT="178" type="application/x-shockwave-flash" pluginspage="http://www.macromedia.com/shockwave/download/index.cgi?P1_Prod_Version=ShockwaveFlash"&gt;&lt;' + '/embed&gt;');}else{document.writeln('&lt;img alt="" src="menu/menu.gif" usemap="#menu" border="0" width="185" height="178"&gt;');}}<xsl:text>
</xsl:text>//</xsl:comment><xsl:text>
</xsl:text></script><noscript><img alt="" src="menu/menu.gif" usemap="#menu" border="0"  width="185" height="178"/></noscript>
</xsl:template>

<!-- special symbols --> 

<xsl:template match="column/text()|i/text()|b/text()|a/text()">
<!-- we change special symbols in text using a recursive named template -->
<xsl:call-template name="rec-text">
  <xsl:with-param name="s" select="."/>
</xsl:call-template>
</xsl:template>

<xsl:template name="rec-text">
<xsl:param name="s"/>
<xsl:param name="countquote" select="0"/>
<xsl:choose>
  <xsl:when test="contains($s, ' -') and not(contains($s, concat(' -', '-')))">
    <xsl:call-template name="rec-text">
      <xsl:with-param name="s" select="concat(substring-before($s, ' -'), ' &#8212;', substring-after($s, ' -'))"/>
    </xsl:call-template>
  </xsl:when>
  <xsl:when test="contains($s, '- ') and not(contains($s, concat('!-', '- ')))">
    <xsl:call-template name="rec-text">
      <xsl:with-param name="s" select="concat(substring-before($s, '- '), '&#8212; ', substring-after($s, '- '))"/>
    </xsl:call-template>
  </xsl:when>
  <xsl:when test="contains($s, $apostrophe)">    
    <xsl:call-template name="rec-text">
      <xsl:with-param name="s" select="concat(substring-before($s, $apostrophe), '&#8217;', substring-after($s, $apostrophe))"/>
    </xsl:call-template>
  </xsl:when>
  <xsl:when test="contains($s, $quote)">
    <xsl:choose>
      <xsl:when test="$countquote=0">
        <xsl:call-template name="rec-text">
          <xsl:with-param name="countquote" select="1"/>
          <xsl:with-param name="s" select="concat(substring-before($s, $quote), '&#8220;', substring-after($s, $quote))"/>
        </xsl:call-template>      
      </xsl:when>
      <xsl:when test="$countquote=1">
        <xsl:call-template name="rec-text">
          <xsl:with-param name="countquote" select="0"/>
          <xsl:with-param name="s" select="concat(substring-before($s, $quote), '&#8221;', substring-after($s, $quote))"/>
        </xsl:call-template>
      </xsl:when>
    </xsl:choose>
  </xsl:when>
  <xsl:otherwise>
    <xsl:value-of select="$s"/>
  </xsl:otherwise>
</xsl:choose>
</xsl:template>

<!-- scrolling for arrows --> 

<xsl:template name="scroll">
<!-- calculating x-coordinate for horizontal scrolling -->
<xsl:variable name="horiz" select="380" saxon:assignable="yes"/>
<xsl:for-each select="//intro/column">
  <saxon:assign name="horiz" select="$horiz + $w-col + $w-btw-col"/>            
</xsl:for-each> 
<saxon:assign name="horiz" select="$horiz + $w-btw-block - $w-btw-col"/>            
<xsl:for-each select="preceding::block">
  <saxon:assign name="horiz" select="$horiz + $w-btw-block - $w-btw-col"/>            
</xsl:for-each> 
<xsl:for-each select="preceding::block/column">
  <saxon:assign name="horiz" select="$horiz + $w-col + $w-btw-col"/>            
</xsl:for-each> 
<xsl:attribute name="onclick">scrollTo(<xsl:value-of select="$horiz"/>,0)</xsl:attribute>
<xsl:attribute name="onmouseover">cursorHand(this)</xsl:attribute>
<xsl:attribute name="onmouseout">window.status=''</xsl:attribute>
<xsl:attribute name="alt">[click to scroll]</xsl:attribute>
</xsl:template>

</xsl:stylesheet>
